//--------------------------------------------------------------//
// Colour Temperature
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Colour Temperature";
   string Category    = "Colour";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
float4 _ColdColour = float4( 0.0, 0.8, 1.0, 1.0 );

float4 _HotColour  = float4( 1.0, 0.5, 0.0, 1.0 );

float Amount
<
   string Description = "Temperature";
   float MinVal = -1.0;
   float MaxVal = 1.0;
> = 0.0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture Input;
sampler InputSampler = sampler_state { Texture = <Input>; };

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );
   float4 ret;

   float oldLum  = ( src.r * 0.3 ) + ( src.g * 0.59 ) + ( src.b * 0.11 );

   if ( Amount > 0 )
   {
      ret = lerp( src, _HotColour, Amount / 5 );
   }
   else
   {
      ret = lerp( _ColdColour, src, 1 + ( Amount / 5 ) );
   }

   float newLum  = ( ret.r * 0.3 ) + ( ret.g * 0.59 ) + ( ret.b * 0.11 );

   // Try to retain a similar luminance
   ret -= ( newLum - oldLum );

   ret.a = src.a;

   return ret;
}


//--------------------------------------------------------------//
// Technique Section for Image Processing Effects
//--------------------------------------------------------------//
technique ColourTemp { pass Single_Pass { PixelShader = compile PROFILE main(); } }
