//--------------------------------------------------------------//
// Header
//
// Lightworks effects have to have a _LwksEffectInfo block
// which defines basic information about the effect (ie. name
// and category). EffectGroup must be "GenericPixelShader".
//--------------------------------------------------------------//
int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Shear";                   // The title
   string Category    = "Samples";                 // Governs the category that the effect appears in in Lightworks
> = 0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//

// For each regular 'texture' declared here, Lightworks adds a matching
// input to your effect (so for a four input effect, you'd need
// to delcare four textures and samplers)

texture Input;

// This is a TWO pass effect, so we need an intermediate texture (
// which stores the results from pass 1, and serves as the input to
// pass 2

texture OutputFromPassOne : RenderColorTarget;


sampler FgSampler = sampler_state
{
   Texture = <Input>;
};

sampler P1OutSampler = sampler_state
{
   Texture = <OutputFromPassOne>;
};

//--------------------------------------------------------------//
// Define parameters here.
//
// The Lightworks application will automatically generate
// sliders/controls for all parameters which do not start
// with a a leading '_' character
//--------------------------------------------------------------//
float Horizontal
<
   string Description = "Horizontal";
   float MinVal       = -1.00;
   float MaxVal       = 1.00;
> = 0.0; // Default value

float Vertical
<
   string Description = "Vertical";
   float MinVal       = -1.00;
   float MaxVal       = 1.00;
> = 0.0; // Default value


#pragma warning ( disable : 3571 )

//--------------------------------------------------------------
// Pixel Shader
//
// This section defines the code which the GPU will
// execute for every pixel in an output image.
//
// Note that pixels are processed out of order, in parallel.
// Using shader model 2.0, so there's a 64 instruction limit -
// use multple passes if you need more.
//--------------------------------------------------------------
float4 ShearHorizontal( float2 xy : TEXCOORD1 ) : COLOR
{
   // Amend the x position based on the user-specified shear amount
   float2 pos = xy;
   pos.x += ( pos.y - 0.5 ) * Horizontal;

   return tex2D( FgSampler, pos );
}

float4 ShearVertical( float2 xy : TEXCOORD1 ) : COLOR
{
   // Amend the x position based on the user-specified shear amount
   float2 pos = xy;
   pos.y += ( pos.x - 0.5 ) * Vertical;

   return tex2D( P1OutSampler, pos );
}

//--------------------------------------------------------------
// Technique
//
// Specifies the order of passes
//--------------------------------------------------------------
technique SampleFxTechnique
{
   pass ShearX
   <
      string Script = "RenderColorTarget0 = OutputFromPassOne;";
   >
   {
      PixelShader = compile PROFILE ShearHorizontal();
   }

   pass ShearY
   {
      PixelShader = compile PROFILE ShearVertical();
   }
}

