//--------------------------------------------------------------//
// Header
//
// Lightworks effects have to have a _LwksEffectInfo block
// which defines basic information about the effect (ie. name
// and category). EffectGroup must be "GenericPixelShader".
//--------------------------------------------------------------//
int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Channel Combiner";        // The title
   string Category    = "Samples";                 // Governs the category that the effect appears in in Lightworks
> = 0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//

// For each 'texture' declared here, Lightworks adds a matching
// input to your effect (so for a four input effect, you'd need
// to delcare four textures and samplers)

texture R;
texture G;
texture B;
texture A;

sampler RSampler = sampler_state { Texture = <R>; };
sampler GSampler = sampler_state { Texture = <G>; };
sampler BSampler = sampler_state { Texture = <B>; };
sampler ASampler = sampler_state { Texture = <A>; };

//--------------------------------------------------------------//
// Define parameters here.
//
// The Lightworks application will automatically generate
// sliders/controls for all parameters which do not start
// with a a leading '_' character
//--------------------------------------------------------------//
float Opacity
<
   string Description = "Opacity";
   float MinVal       = 0.0;
   float MaxVal       = 1.0;
> = 1.0; // Default value

#pragma warning ( disable : 3571 )

//--------------------------------------------------------------
// Pixel Shader
//
// This section defines the code which the GPU will
// execute for every pixel in an output image.
//
// Note that pixels are processed out of order, in parallel.
// Using shader model 2.0, so there's a 64 instruction limit -
// use multple passes if you need more.
//--------------------------------------------------------------
float4 CombineRGBA( float2 xy : TEXCOORD1 ) : COLOR
{
   return float4( tex2D( RSampler, xy ).r,
                  tex2D( GSampler, xy ).g,
                  tex2D( BSampler, xy ).b,
                  tex2D( ASampler, xy ).a * Opacity );
}

//--------------------------------------------------------------
// Technique
//
// Specifies the order of passes (we only have a single pass, so
// there's not much to do)
//--------------------------------------------------------------
technique SampleFxTechnique
{
   pass SinglePass
   {
      PixelShader = compile PROFILE CombineRGBA();
   }
}

