//--------------------------------------------------------------//
// Header
//
// Lightworks effects have to have a _LwksEffectInfo block
// which defines basic information about the effect (ie. name
// and category). EffectGroup must be "GenericPixelShader".
//--------------------------------------------------------------//
int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Channel Extractor";       // The title
   string Category    = "Samples";                 // Governs the category that the effect appears in in Lightworks
> = 0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//

// For each 'texture' declared here, Lightworks adds a matching
// input to your effect (so for a four input effect, you'd need
// to delcare four textures and samplers)

texture Input;

sampler FgSampler = sampler_state
{
   Texture = <Input>;
};

//--------------------------------------------------------------//
// Define parameters here.
//
// The Lightworks application will automatically generate
// sliders/controls for all parameters which do not start
// with a a leading '_' character
//--------------------------------------------------------------//

// If Lightworks encounters a parameter called 'SetTechnique', it will be shown on the FX
// panel like any other parameter, but it will also implicitly choose an execution technique
// to match the user-selection (eg. if the user chooses 'Green' from the drop-down list,
// Lightworks will arrange for the 'Green' technique (at the bottom of this file) to be executed
// when the effect is rendered)
int SetTechnique
<
   string Description = "Channel";
   string Enum = "Red,Green,Blue,Alpha";
> = 0;

#pragma warning ( disable : 3571 )

//--------------------------------------------------------------
// Pixel Shader
//
// This section defines the code which the GPU will
// execute for every pixel in an output image.
//
// Note that pixels are processed out of order, in parallel.
// Using shader model 2.0, so there's a 64 instruction limit -
// use multple passes if you need more.
//--------------------------------------------------------------
float4 ExtractRed( float2 xy : TEXCOORD1 ) : COLOR
{
   float4 pixel = tex2D( FgSampler, xy );
   return float4( pixel.r, pixel.r, pixel.r, 1 );
}

float4 ExtractGreen( float2 xy : TEXCOORD1 ) : COLOR
{
   float4 pixel = tex2D( FgSampler, xy );
   return float4( pixel.g, pixel.g, pixel.g, 1 );
}

float4 ExtractBlue( float2 xy : TEXCOORD1 ) : COLOR
{
   float4 pixel = tex2D( FgSampler, xy );
   return float4( pixel.b, pixel.b, pixel.b, 1 );
}

float4 ExtractAlpha( float2 xy : TEXCOORD1 ) : COLOR
{
   float4 pixel = tex2D( FgSampler, xy );
   return float4( pixel.a, pixel.a, pixel.a, 1 );
}

//--------------------------------------------------------------
// Techniques
//--------------------------------------------------------------
technique Red
{
   pass SinglePass
   {
      PixelShader = compile PROFILE ExtractRed();
   }
}

technique Green
{
   pass SinglePass
   {
      PixelShader = compile PROFILE ExtractGreen();
   }
}

technique Blue
{
   pass SinglePass
   {
      PixelShader = compile PROFILE ExtractBlue();
   }
}

technique Alpha
{
   pass SinglePass
   {
      PixelShader = compile PROFILE ExtractAlpha();
   }
}


