//--------------------------------------------------------------//
// StereoscopicAdjust.fx
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Null";
   string Category    = "Null";
> = 0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture Input;
sampler InputSampler = sampler_state
{
   Texture   = <Input>;
};

//--------------------------------------------------------------//
// Params/Constants
//--------------------------------------------------------------//
float HueShift = 0.0;
float SatMult  = 1.0;
float LumMult  = 1.0;

const float _PI_DIV_180 = 3.14159265/180.0;

const float3x3 _kRGBToYIQ =
{
   0.299,  0.587,  0.114,
   0.596, -0.274, -0.321,
   0.211, -0.523,  0.311,
};

const float3x3 _kYIQToRGB =
{
   1,  0.956,  0.621,
   1, -0.272, -0.647,
   1, -1.107,  1.705,
};

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 colourCorrect_main( float2 xy : TEXCOORD0 ) : COLOR
{
   float4 rgba = tex2D( InputSampler, xy );

   // See http://beesbuzz.biz/code/hsv_color_transforms.php

   float LumSat = LumMult * SatMult;
   float HuePie = HueShift * _PI_DIV_180;

   float VSU = LumSat * cos( HuePie );
   float VSW = LumSat * sin( HuePie );

   float3x3 adjustment =
   {
      LumMult, 0,    0,
      0,         VSU, -VSW,
      0,         VSW,  VSU
   };

   rgba.rgb = mul( _kYIQToRGB, mul( mul( _kRGBToYIQ, rgba.rgb ), adjustment ) );

   return rgba;
}

//--------------------------------------------------------------
technique ColourCorrect { pass Single_Pass { PixelShader = compile PROFILE colourCorrect_main(); } }
