//--------------------------------------------------------------//
// TextEffects.fx
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "Internal";
> = 0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture Input;
sampler LinearSampler = sampler_state
{
   Texture = <Input>;
   MINFILTER = LINEAR;
   MAGFILTER = LINEAR;
};

float4 ShadowColour   = float4( 0,0,0,0.7 );
float4 FaceColour1    = float4( 1,1,1,1 );
float4 FaceColour2    = float4( 1,1,1,1 );
float  PixelWidth     = 1.0;
float  PixelHeight    = 1.0;
float  ShadowSoftness = 0.7;
float  OutlineSize    = 1.0;
float  Opacity        = 1.0;
float4 OutlineColour  = float4( 0,0,0,1 );

float _kMidPos = 0.6;
float _MaxOutlineSize = 5.0;

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 verticalGradient( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( LinearSampler, xy1 );

   float4 output = lerp( FaceColour1, FaceColour2, ( xy1.y * 2 ) - 0.5 );
   output.a = src.r * FaceColour1.a;

   return output;
}
//--------------------------------------------------------------//
float4 sheen( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 output;
   float4 src = tex2D( LinearSampler, xy1 );

   if ( xy1.y < _kMidPos )
      output = lerp( FaceColour1, FaceColour2, ( ( xy1.y / _kMidPos ) - 0.5 ) * 2 );
   else
      output = lerp( FaceColour1, FaceColour2, ( ( ( 1 - xy1.y ) / ( 1.0 - _kMidPos ) ) - 0.5 ) * 2 );

   output.a = src.r * FaceColour1.a;

   return output;
}
//--------------------------------------------------------------//
float4 flatColour( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( LinearSampler, xy1 );

   float4 output = FaceColour1;
   output.a = src.r * FaceColour1.a;

   return output;
}

//--------------------------------------------------------------//
float4 shadow( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( LinearSampler, xy1 );

   float4 output = ShadowColour;
   output.a = src.r * ShadowColour.a;

   return output;
}
//--------------------------------------------------------------//
float4 clearTypeBlend( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( LinearSampler, xy1 );

   float4 output = FaceColour1;
   output.a = ( ( src.r + src.g + src.b ) / 3.0 ) * FaceColour1.a;

   return output;
}

float gaussian[] =
{
   1.0 / 4096.0,
   12.0 / 4096.0,
   66.0 / 4096.0,
   220.0 / 4096.0,
   495.0 / 4096.0,
   792.0 / 4096.0,
   924.0 / 4096.0
};

//--------------------------------------------------------------//
float4 shadowPassOne( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float one   = PixelWidth * ShadowSoftness;
   float two   = one * 2;
   float half  = one / 2;

   float tap1  = xy1.x + one + half;
   float tap2  = tap1 + two;
   float tap3  = tap2 + two;
   float tap4  = tap3 + two;
   float tap5  = tap4 + two;
   float tap6  = tap5 + two;
   float ntap1 = xy1.x - ( one + half );
   float ntap2 = ntap1 - two;
   float ntap3 = ntap2 - two;
   float ntap4 = ntap3 - two;
   float ntap5 = ntap4 - two;
   float ntap6 = ntap5 - two;

   float4 pixel = tex2D( LinearSampler, xy1 );
   float4 blurred = pixel * gaussian[ 6 ];

   blurred += tex2D( LinearSampler, float2( tap1,  xy1.y ) ) * gaussian[ 5 ];
   blurred += tex2D( LinearSampler, float2( tap2,  xy1.y ) ) * gaussian[ 4 ];
   blurred += tex2D( LinearSampler, float2( tap3,  xy1.y ) ) * gaussian[ 3 ];
   blurred += tex2D( LinearSampler, float2( tap4,  xy1.y ) ) * gaussian[ 2 ];
   blurred += tex2D( LinearSampler, float2( tap5,  xy1.y ) ) * gaussian[ 1 ];
   blurred += tex2D( LinearSampler, float2( tap6,  xy1.y ) ) * gaussian[ 0 ];

   blurred += tex2D( LinearSampler, float2( ntap1, xy1.y ) ) * gaussian[ 5 ];
   blurred += tex2D( LinearSampler, float2( ntap2, xy1.y ) ) * gaussian[ 4 ];
   blurred += tex2D( LinearSampler, float2( ntap3, xy1.y ) ) * gaussian[ 3 ];
   blurred += tex2D( LinearSampler, float2( ntap4, xy1.y ) ) * gaussian[ 2 ];
   blurred += tex2D( LinearSampler, float2( ntap5, xy1.y ) ) * gaussian[ 1 ];
   blurred += tex2D( LinearSampler, float2( ntap6, xy1.y ) ) * gaussian[ 0 ];

   return blurred.r;
}

//--------------------------------------------------------------//
float4 shadowPassTwo( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float one   = PixelHeight * ShadowSoftness;
   float two   = one * 2;
   float half  = one / 2;

   float tap1  = xy1.y + one + half;
   float tap2  = tap1 + two;
   float tap3  = tap2 + two;
   float tap4  = tap3 + two;
   float tap5  = tap4 + two;
   float tap6  = tap5 + two;
   float ntap1 = xy1.y - ( one + half );
   float ntap2 = ntap1 - two;
   float ntap3 = ntap2 - two;
   float ntap4 = ntap3 - two;
   float ntap5 = ntap4 - two;
   float ntap6 = ntap5 - two;

   float4 pixel = tex2D( LinearSampler, xy1 );
   float4 blurred = pixel * gaussian[ 6 ];

   blurred += tex2D( LinearSampler, float2( xy1.x,  tap1 ) ) * gaussian[ 5 ];
   blurred += tex2D( LinearSampler, float2( xy1.x,  tap2 ) ) * gaussian[ 4 ];
   blurred += tex2D( LinearSampler, float2( xy1.x,  tap3 ) ) * gaussian[ 3 ];
   blurred += tex2D( LinearSampler, float2( xy1.x,  tap4 ) ) * gaussian[ 2 ];
   blurred += tex2D( LinearSampler, float2( xy1.x,  tap5 ) ) * gaussian[ 1 ];
   blurred += tex2D( LinearSampler, float2( xy1.x,  tap6 ) ) * gaussian[ 0 ];

   blurred += tex2D( LinearSampler, float2( xy1.x, ntap1 ) ) * gaussian[ 5 ];
   blurred += tex2D( LinearSampler, float2( xy1.x, ntap2 ) ) * gaussian[ 4 ];
   blurred += tex2D( LinearSampler, float2( xy1.x, ntap3 ) ) * gaussian[ 3 ];
   blurred += tex2D( LinearSampler, float2( xy1.x, ntap4 ) ) * gaussian[ 2 ];
   blurred += tex2D( LinearSampler, float2( xy1.x, ntap5 ) ) * gaussian[ 1 ];
   blurred += tex2D( LinearSampler, float2( xy1.x, ntap6 ) ) * gaussian[ 0 ];

   blurred.rgb = ShadowColour.rgb;
   blurred.a = pow( blurred.a, 0.3 );
   blurred.a *= ShadowColour.a * 3;


   return blurred;
}

//--------------------------------------------------------------//
float4 outlinePassOne( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float one   = PixelWidth * OutlineSize;
   float two   = one * 2;
   float half  = one / 2;

   float tap1  = xy1.x + one + half;
   float tap2  = tap1 + two;
   float tap3  = tap2 + two;
   float tap4  = tap3 + two;
   float tap5  = tap4 + two;
   float tap6  = tap5 + two;
   float ntap1 = xy1.x - ( one + half );
   float ntap2 = ntap1 - two;
   float ntap3 = ntap2 - two;
   float ntap4 = ntap3 - two;
   float ntap5 = ntap4 - two;
   float ntap6 = ntap5 - two;

   float4 pixel = tex2D( LinearSampler, xy1 );
   float4 blurred = pixel * gaussian[ 6 ];

   blurred += tex2D( LinearSampler, float2( tap1,  xy1.y ) ) * gaussian[ 5 ];
   blurred += tex2D( LinearSampler, float2( tap2,  xy1.y ) ) * gaussian[ 4 ];
   blurred += tex2D( LinearSampler, float2( tap3,  xy1.y ) ) * gaussian[ 3 ];
   blurred += tex2D( LinearSampler, float2( tap4,  xy1.y ) ) * gaussian[ 2 ];
   blurred += tex2D( LinearSampler, float2( tap5,  xy1.y ) ) * gaussian[ 1 ];
   blurred += tex2D( LinearSampler, float2( tap6,  xy1.y ) ) * gaussian[ 0 ];

   blurred += tex2D( LinearSampler, float2( ntap1, xy1.y ) ) * gaussian[ 5 ];
   blurred += tex2D( LinearSampler, float2( ntap2, xy1.y ) ) * gaussian[ 4 ];
   blurred += tex2D( LinearSampler, float2( ntap3, xy1.y ) ) * gaussian[ 3 ];
   blurred += tex2D( LinearSampler, float2( ntap4, xy1.y ) ) * gaussian[ 2 ];
   blurred += tex2D( LinearSampler, float2( ntap5, xy1.y ) ) * gaussian[ 1 ];
   blurred += tex2D( LinearSampler, float2( ntap6, xy1.y ) ) * gaussian[ 0 ];

   return blurred.r;
}

//--------------------------------------------------------------//
float4 outlinePassTwo( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float one   = PixelHeight * OutlineSize;
   float two   = one * 2;
   float half  = one / 2;

   float tap1  = xy1.y + one + half;
   float tap2  = tap1 + two;
   float tap3  = tap2 + two;
   float tap4  = tap3 + two;
   float tap5  = tap4 + two;
   float tap6  = tap5 + two;
   float ntap1 = xy1.y - ( one + half );
   float ntap2 = ntap1 - two;
   float ntap3 = ntap2 - two;
   float ntap4 = ntap3 - two;
   float ntap5 = ntap4 - two;
   float ntap6 = ntap5 - two;

   float4 pixel = tex2D( LinearSampler, xy1 );
   float4 blurred = pixel * gaussian[ 6 ];

   blurred += tex2D( LinearSampler, float2( xy1.x,  tap1 ) ) * gaussian[ 5 ];
   blurred += tex2D( LinearSampler, float2( xy1.x,  tap2 ) ) * gaussian[ 4 ];
   blurred += tex2D( LinearSampler, float2( xy1.x,  tap3 ) ) * gaussian[ 3 ];
   blurred += tex2D( LinearSampler, float2( xy1.x,  tap4 ) ) * gaussian[ 2 ];
   blurred += tex2D( LinearSampler, float2( xy1.x,  tap5 ) ) * gaussian[ 1 ];
   blurred += tex2D( LinearSampler, float2( xy1.x,  tap6 ) ) * gaussian[ 0 ];

   blurred += tex2D( LinearSampler, float2( xy1.x, ntap1 ) ) * gaussian[ 5 ];
   blurred += tex2D( LinearSampler, float2( xy1.x, ntap2 ) ) * gaussian[ 4 ];
   blurred += tex2D( LinearSampler, float2( xy1.x, ntap3 ) ) * gaussian[ 3 ];
   blurred += tex2D( LinearSampler, float2( xy1.x, ntap4 ) ) * gaussian[ 2 ];
   blurred += tex2D( LinearSampler, float2( xy1.x, ntap5 ) ) * gaussian[ 1 ];
   blurred += tex2D( LinearSampler, float2( xy1.x, ntap6 ) ) * gaussian[ 0 ];

   blurred.rgb = OutlineColour.rgb;
   blurred.a = pow( blurred.a, 0.3 );
   blurred.a *= OutlineColour.a * 3;

   return blurred;
}

//--------------------------------------------------------------//
float4 blend( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 pixel = tex2D( LinearSampler, xy1 );
   pixel.a *= Opacity;

   return pixel;
}

//--------------------------------------------------------------//
// Technique Section for Image Processing Effects
//--------------------------------------------------------------//
technique FlatColour       { pass p1 { PixelShader = compile PROFILE flatColour(); } }
technique VerticalGradient { pass p1 { PixelShader = compile PROFILE verticalGradient(); } }
technique Sheen            { pass p1 { PixelShader = compile PROFILE sheen(); } }
technique Shadow           { pass p1 { PixelShader = compile PROFILE shadow(); } }
technique ShadowPass1      { pass p1 { PixelShader = compile PROFILE shadowPassOne(); } }
technique ShadowPass2      { pass p1 { PixelShader = compile PROFILE shadowPassTwo(); } }

technique OutlinePass1     { pass p1 { PixelShader = compile PROFILE outlinePassOne(); } }
technique OutlinePass2     { pass p1 { PixelShader = compile PROFILE outlinePassTwo(); } }

technique ClearTypeBlend   { pass p1 { PixelShader = compile PROFILE clearTypeBlend(); } }

technique Blend            { pass p1 { PixelShader = compile PROFILE blend(); } }

