//-------------------------------------------------------------------------------------------------
// ColourConvert.fx
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//-------------------------------------------------------------------------------------------------

// FLOATING POINT MATH
float4 ps_UnpackMatrox10Bit_BaseOnly( float2 TexUV0 : TEXCOORD1 ) : COLOR0
{
   // The source surface must be in A8R8G8B8 format, wrapping Matrox-format data. The rendertarget is expected
   // to be in X16R16G16B16. The output data will be in 16-bit YUYV format, with values normalised to the range 0.0->1.0.
   //

   // Matrox 10 bit YCrCb 4:2:2 format is packed as follows:
   //
   //               |------------------------------------------ BASE BLOCK --------------------||---------------- EXTRAS BLOCK ----------------|
   // DWORD offset  |----------- 0 ---------||---------- 1 ---------|.. |---------- 7 ---------||---------- 8 ---------||---------- 9 ---------|
   // Byte Offset   0      1     2     3     4     5     6     7     .. 28    29    30    31    32    33    34    35    36    37    38    39
   // Value         Y0     Cb0   Y1    Cr0   Y2    Cb2   Y3    Cr2   .. Y14   Cb14  Y15   Cr14  Packed high-order 2 bits of preceeding pixels, order TBC

   const int   kBlockSizeInComponentValuesI = 32;

   const int   kBaseBlockSizeInDWORDsI = kBlockSizeInComponentValuesI / 4;       // eg, 8
   const float kBaseBlockSizeInDWORDsF = (float)kBaseBlockSizeInDWORDsI;         //     8.0

   const int   kExtrasBlockSizeInDWORDsI = kBlockSizeInComponentValuesI / 16;    //     2
   const float kExtrasBlockSizeInDWORDsF = (float)kExtrasBlockSizeInDWORDsI;     //     2.0

   const int   kTotalBlockSizeInDWORDsI = kBaseBlockSizeInDWORDsI + kExtrasBlockSizeInDWORDsI;  // 10
   const float kTotalBlockSizeInDWORDsF = (float)kTotalBlockSizeInDWORDsI;       // 10

   // TEXCOORD0: y = output row 0.0->1.0 format. x = output column, 0 - ( SourceImageWidth - 1 ) format.

   float phase = fmod( TexUV0.x, kBaseBlockSizeInDWORDsF );
   float block = floor( TexUV0.x / kBaseBlockSizeInDWORDsF );

   // Determine the x-coord where the four 8-bit values will come from:
   float xDWORDPixelOffset = floor( ( block * kTotalBlockSizeInDWORDsF ) + phase );

   TexUV0.x = ( xDWORDPixelOffset + 0.5 ) / pSourceImageTotalWidthInDWORDs;

   // now 16 bit value scaled to 0..1 instead of 8 bit
   float4 base;
   base = tex2D( SourceImageSampler, TexUV0 );

   return base * 255.0f * 256.0f / 65535.0f;
   // approximately 16 instruction slots used (1 texture, 15 arithmetic)
};

// FLOATING POINT MATH
float4 ps_UnpackMatrox10Bit( float2 TexUV0 : TEXCOORD1 ) : COLOR0
{
   // The source surface must be in A8R8G8B8 format, wrapping Matrox-format data. The rendertarget is expected
   // to be in X16R16G16B16. The output data will be in 16-bit YUYV format, with values normalised to the range 0.0->1.0.
   //

   // Matrox 10 bit YCrCb 4:2:2 format is packed as follows:
   //
   //               |------------------------------------------ BASE BLOCK --------------------||---------------- EXTRAS BLOCK ----------------|
   // DWORD offset  |----------- 0 ---------||---------- 1 ---------|.. |---------- 7 ---------||---------- 8 ---------||---------- 9 ---------|
   // Byte Offset   0      1     2     3     4     5     6     7     .. 28    29    30    31    32    33    34    35    36    37    38    39
   // Value         Y0     Cb0   Y1    Cr0   Y2    Cb2   Y3    Cr2   .. Y14   Cb14  Y15   Cr14  Packed high-order 2 bits of preceeding pixels, order TBC

   const int   kBlockSizeInComponentValuesI = 32;

   const int   kBaseBlockSizeInDWORDsI = kBlockSizeInComponentValuesI / 4;       // eg, 8
   const float kBaseBlockSizeInDWORDsF = (float)kBaseBlockSizeInDWORDsI;         //     8.0

   const int   kExtrasBlockSizeInDWORDsI = kBlockSizeInComponentValuesI / 16;    //     2
   const float kExtrasBlockSizeInDWORDsF = (float)kExtrasBlockSizeInDWORDsI;     //     2.0

   const int   kTotalBlockSizeInDWORDsI = kBaseBlockSizeInDWORDsI + kExtrasBlockSizeInDWORDsI;  // 10
   const float kTotalBlockSizeInDWORDsF = (float)kTotalBlockSizeInDWORDsI;       // 10

   // TEXCOORD0: y = output row 0.0->1.0 format. x = output column, 0 - ( SourceImageWidth - 1 ) format.

   float phase = fmod( TexUV0.x, kBaseBlockSizeInDWORDsF );
   float block = floor( TexUV0.x / kBaseBlockSizeInDWORDsF );

   // Determine the x-coord where the four 8-bit values will come from:
   float xDWORDPixelOffset = floor( ( block * kTotalBlockSizeInDWORDsF ) + phase );

   // Determine the x-coord where of the DWORD the byte of 2-bit extras will come from
   float xDWORDExtrasOffset = ( block * kTotalBlockSizeInDWORDsF ) + kBaseBlockSizeInDWORDsF + ( phase / 4 );

   // We'll end up with a float4 of extras wrapping 4 bytes. We need just one byte of that. Determine the
   // vector index into the float4 which we need to use.
   float xExtrasVectorIndex = fmod( phase, 4.0 );

   TexUV0.x = ( xDWORDPixelOffset + 0.5 ) / pSourceImageTotalWidthInDWORDs;

   // start with the 8 bit value from the input
   float4 base;
   base = tex2D( SourceImageSampler, TexUV0 );

   // set extras data position
   TexUV0.x = ( xDWORDExtrasOffset + 0.1 ) / pSourceImageTotalWidthInDWORDs;

   // sample extra 2 bitvalue from the source block
   float4 extras;
   extras.argb = tex2D( SourceImageSampler, TexUV0 ).abgr;

   // extract required byte as 0..255
   float extraB = floor( ( 255.0 * extras[ xExtrasVectorIndex ] ) + 0.5 );

   // base[n] contains the most significant 8 bits, with the two additional LSBs coming from two bits of extraB.

   // We need to get the four 2-bit numbers out of extraB. We don't have bitwise operators like &, >> or <<, so
   // this is a bit messy. Also, integers may well be emulated using floating point in the GPU, so we can't rely
   // on 'normal' integer rounding and overflow behaviour.

   float bb76 = floor( ( extraB / 64.0 ) + ( 0.5 / 64.0 ) );
   float bb54 = floor( ( ( extraB - ( bb76 * 64.0 ) ) / 16.0 ) + ( 0.5 / 16.0 ) );
   float bb32 = floor( ( ( extraB - ( ( bb76 * 64.0 ) + ( bb54 * 16.0 ) ) ) / 4.0 ) + ( 0.5 / 4.0 ) );
   float bb10 = floor( ( extraB - ( ( bb76 * 64.0 ) + ( bb54 * 16.0 )  + ( bb32 * 4.0 ) ) ) + 0.5 );

   // create array of 4 2 bit values corresponding to Y0 Cr0 Y1 Cb0
   float4 bbf;
   bbf[0] = bb10;	// Y0
   bbf[1] = bb32;	// Cb0
   bbf[2] = bb54;	// Y1
   bbf[3] = bb76;	// Cr0

   float4 ret;

   // Y values are offset by 16/256
   // Cr/Cb values are offset by 128/256

   // merge in bottom 2 bits and scale to 0..1 from 0..1023?
   ret = ( base * 255.0f * 4.0f + bbf );

   // scale 10bit value to 16 bit
   return ret * 64.0f / 65535.0f;
   // approximately 58 instruction slots used (2 texture, 56 arithmetic)
};

float4 ps_PackMatrox10Bit( float2 TexUV1 : TEXCOORD1 ) : COLOR0
{
/*
   - Called once per output pixel.
   - 8 calls for outputting high 8 bits as YCrYCb
   - 2 calls for outputting extra bits, each call referencing 4 source pixels ?? Can we do enough processing ??
     ^ rough out the above, not caring where the numbers come from.
*/

   // TEXCOORD0: y = output row 0.0->1.0 format. x = output column, 0 - ( pSourceImageTotalWidthInDWORDs - 1 ) format.
   const int   kBlockSizeInComponentValuesI = 32;

   const int   kBaseBlockSizeInDWORDsI = kBlockSizeInComponentValuesI / 4;       // eg, 8
   const float kBaseBlockSizeInDWORDsF = (float)kBaseBlockSizeInDWORDsI;         //     8.0

   const int   kExtrasBlockSizeInDWORDsI = kBlockSizeInComponentValuesI / 16;    //     2
   const float kExtrasBlockSizeInDWORDsF = (float)kExtrasBlockSizeInDWORDsI;     //     2.0

   const int   kTotalBlockSizeInDWORDsI = kBaseBlockSizeInDWORDsI + kExtrasBlockSizeInDWORDsI;  // 10
   const float kTotalBlockSizeInDWORDsF = (float)kTotalBlockSizeInDWORDsI;       // 10

   float x = TexUV1.x * DestImageWidth;

   float block = floor( ( x / kTotalBlockSizeInDWORDsF ) + ( 0.5 / kTotalBlockSizeInDWORDsF ) );
   float phase = floor( x - ( block * kTotalBlockSizeInDWORDsF ) );

   float4 ret;
   float4 ret2;

   // phase from 0..9, so for 8,9 outputting extras
   if ( phase < kBaseBlockSizeInDWORDsF )
   {
      // Need to write out some 8-bit wide high bytes

      // NOTE: Hmm. Might just be able to output sourcePixel without any
      //       adjustments - depends how the 16 -> 8 bit rounding might work.
      float2 uv;
      uv.y = TexUV1.y;

      // calculate pixel to sample (in range 0..1
      // (1) rounding required for HAL, not for s/w
      uv.x = ( ( block * kBaseBlockSizeInDWORDsF ) + phase + 0.1 ) / pSourceImageTotalWidthInDWORDs;

      // round to 8bit
      ret = floor( 256.0f * tex2D( SourceImageSampler, uv ) ) / 256.0f * 256.0f / 255.0f;
	  ret2 = ret;
   }
   else
   {
      // Need to write out some 2-bit extras
//phase = 8,9, offset = 0,4
      float offset = ( phase - 8.0 ) * 4.0;  // We want to look at either the first 4 or second 4 DWORDs in the
                                             // block, depending on whether we're outputting the first or second
                                             // DWORD of extra bits - determined by phase.

      // (2) rounding required for HAL, not for s/w
      float blockStart = ( block * kBaseBlockSizeInDWORDsF ) + offset + 0.5;

      float4x2 uv;
// sample the 4 input pixel values

      uv._m01_m11_m21_m31 = TexUV1.y;
      uv._m00_m10_m20_m30 = blockStart;

      const float4 kAdds = { 0, 1, 2, 3 };

      uv._m00_m10_m20_m30 += kAdds;
      uv._m00_m10_m20_m30 /= pSourceImageTotalWidthInDWORDs;

      row_major float4x4 pixel_values;

      //pixel_values[ 0 ].argb = 256* tex2D( SourceImageSampler, uv[0] ).abgr;
      //pixel_values[ 1 ].argb = 256* tex2D( SourceImageSampler, uv[1] ).abgr;
      //pixel_values[ 2 ].argb = 256* tex2D( SourceImageSampler, uv[2] ).abgr;
      //pixel_values[ 3 ].argb = 256* tex2D( SourceImageSampler, uv[3] ).abgr;

      pixel_values[ 0 ] = 256.0f* tex2D( SourceImageSampler, uv[0] );
      pixel_values[ 1 ] = 256.0f* tex2D( SourceImageSampler, uv[1] );
      pixel_values[ 2 ] = 256.0f* tex2D( SourceImageSampler, uv[2] );
      pixel_values[ 3 ] = 256.0f* tex2D( SourceImageSampler, uv[3] );

      // e.g. 1,2,3,4.xxxx
      // get fractional part (.xxxx)
      pixel_values[ 0 ] = round( frac( pixel_values[ 0 ] ) ) / 4.0;
      pixel_values[ 1 ] = round( frac( pixel_values[ 1 ] ) ) / 4.0;
      pixel_values[ 2 ] = round( frac( pixel_values[ 2 ] ) ) / 4.0;
      pixel_values[ 3 ] = round( frac( pixel_values[ 3 ] ) ) / 4.0;

      // combine and scale to 0..255
      const float4 k2BitMulVec = { 4.0f/255.0f, 16.0f/255.0f, 64.0f/255.0f, 256.0f/255.0f };

	  // sum the input samples
      ret[0] = dot( pixel_values[0], k2BitMulVec );
      ret[1] = dot( pixel_values[1], k2BitMulVec );
      ret[2] = dot( pixel_values[2], k2BitMulVec );
      ret[3] = dot( pixel_values[3], k2BitMulVec );

	  ret2.argb = ret.abgr;
   }

   return ret2;
}
