//--------------------------------------------------------------//
// Blend.fx
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Blend";
   string Category    = "Mixes";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
int SetTechnique
<
   string Description = "Method";
   string Enum = "In Front,Add,Subtract,Multiply,Screen,Overlay,Soft Light,Hard Light,Exclusion,Lighten,Darken,Average,Difference,Colour,Luminosity,Dodge,Burn";
> = 0;

float Opacity
<
   string Description = "Fg Opacity";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 1.0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture fg;
texture bg;
sampler FgSampler = sampler_state { Texture = <fg>; };
sampler BgSampler = sampler_state { Texture = <bg>; };

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//

float4 InFront_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   float4 ret = lerp( bg, fg, fg.a * Opacity );
   ret.a = 1.0;

   return ret;
}
//--------------------------------------------------------------//
float4 Add_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   float4 ret = lerp( bg, clamp( fg + bg, 0.0, 1.0 ), fg.a * Opacity );
   ret.a = 1.0;

   return ret;
}
//--------------------------------------------------------------//
float4 Subtract_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   float4 ret = lerp( bg, clamp( fg - bg, 0.0, 1.0 ), fg.a * Opacity );
   ret.a = 1.0;

   return ret;
}
//--------------------------------------------------------------//
float4 Multiply_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   float4 ret = lerp( bg, bg * fg, fg.a * Opacity );
   ret.a = 1.0;

   return ret;
}
//--------------------------------------------------------------//
float4 Screen_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   float4 ret = 1.0 -( ( 1.0 - fg ) * ( 1.0 - bg ) );
   ret = lerp( bg, ret, fg.a * Opacity );
   ret.a = 1.0;

   return ret;
}
//--------------------------------------------------------------//
float4 Overlay_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   float4 result;

   if ( bg.r < 0.5 )
      result.r = 2.0 * fg.r * bg.r;
   else
      result.r = 1.0 - ( 2.0 * ( 1.0 - fg.r ) * ( 1.0 - bg.r ) );

   if ( bg.g < 0.5 )
      result.g = 2.0 * fg.g * bg.g;
   else
      result.g = 1.0 - ( 2.0 * ( 1.0 - fg.g ) * ( 1.0 - bg.g ) );

   if ( bg.b < 0.5 )
      result.b = 2.0 * fg.b * bg.b;
   else
      result.b = 1.0 - ( 2.0 * ( 1.0 - fg.b ) * ( 1.0 - bg.b ) );

   result.rgb = lerp( bg.rgb, result.rgb, fg.a * Opacity );
   result.a   = 1.0;

   return result;
}
//--------------------------------------------------------------//
float4 SoftLight_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   float4 ret = ( 1.0 - bg ) * ( fg * bg ) + ( bg * ( 1.0 - ( ( 1.0 - bg ) * ( 1.0 - fg ) ) ) );

   ret = lerp( bg, ret, fg.a * Opacity );
   ret.a = 1.0;

   return ret;
}
//--------------------------------------------------------------//
float4 Hardlight_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   float4 result;

   if ( fg.r < 0.5 )
      result.r = 2.0 * fg.r * bg.r;
   else
      result.r = 1.0 - ( 2.0 * ( 1.0 - fg.r ) * ( 1.0 - bg.r ) );

   if ( fg.g < 0.5 )
      result.g = 2.0 * fg.g * bg.g;
   else
      result.g = 1.0 - ( 2.0 * ( 1.0 - fg.g ) * ( 1.0 - bg.g ) );

   if ( fg.b < 0.5 )
      result.b = 2.0 * fg.b * bg.b;
   else
      result.b = 1.0 - ( 2.0 * ( 1.0 - fg.b ) * ( 1.0 - bg.b ) );

   result.rgb = lerp( bg.rgb, result.rgb, fg.a * Opacity );
   result.a = 1.0;

   return result;
}
//--------------------------------------------------------------//
float4 Exclusion_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   float4 ret = fg + bg - ( 2.0 * fg * bg );
   ret = lerp( bg, ret, fg.a * Opacity );
   ret.a = 1.0;

   return ret;
}
//--------------------------------------------------------------//
float4 Lighten_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   float4 ret = lerp( bg, max( fg, bg ), fg.a * Opacity );
   ret.a = 1.0;

   return ret;
}
//--------------------------------------------------------------//
float4 Darken_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   float4 ret = min( fg, bg );
   ret = lerp( bg, ret, fg.a * Opacity );
   ret.a = 1.0;

   return ret;
}
//--------------------------------------------------------------//
float4 Average_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   float4 ret = ( fg + bg ) / 2.0;
   ret = lerp( bg, ret, fg.a * Opacity );
   ret.a = 1.0;

   return ret;
}
//--------------------------------------------------------------//
float4 Difference_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   float4 ret = abs( fg - bg );
   ret = lerp( bg, ret, fg.a * Opacity );
   ret.a = 1.0;

   return ret;
}
/*
//--------------------------------------------------------------//
float4 Hue_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   return fg;
}
//--------------------------------------------------------------//
float4 Saturation_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   return fg;
}
*/
//--------------------------------------------------------------//
float4 Color_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 ret;
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   // Calc source luminance but use dest colour..
   float dstY  = ( 0.257 * bg.r ) + ( 0.504 * bg.g ) + ( 0.098 * bg.b ) + 0.0625;
   float srcCr = ( 0.439 * fg.r ) - ( 0.368 * fg.g ) - ( 0.071 * fg.b ) + 0.5;
   float srcCb = ( -0.148 * fg.r ) -( 0.291 * fg.g ) + ( 0.439 * fg.b ) + 0.5;

   // Convert to RGB..
   float YBit = 1.164 * ( dstY - 0.0625 );
   ret.r = ( YBit + 1.596*(srcCr - 0.5) );
   ret.g = ( YBit - 0.813*(srcCr - 0.5) - 0.391*(srcCb - 0.5) );
   ret.b = ( YBit + 2.018*(srcCb - 0.5) );

   ret.rgb = lerp( bg.rgb, clamp( ret.rgb, 0.0, 1.0 ).rgb, fg.a * Opacity );
   ret.a = 1.0;

   return ret;
}
//--------------------------------------------------------------//
float4 Luminosity_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 ret;
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   // Calc source luminance but use dest colour..
   float srcY  = ( 0.257 * fg.r ) + ( 0.504 * fg.g ) + ( 0.098 * fg.b ) + 0.0625;
   float dstCr = ( 0.439 * bg.r ) - ( 0.368 * bg.g ) - ( 0.071 * bg.b ) + 0.5;
   float dstCb = ( -0.148 * bg.r ) -( 0.291 * bg.g ) + ( 0.439 * bg.b ) + 0.5;

   // Convert to RGB..
   float YBit = 1.164 * (srcY - 0.0625);
   ret.r = ( YBit + 1.596*(dstCr - 0.5) );
   ret.g = ( YBit - 0.813*(dstCr - 0.5) - 0.391*(dstCb - 0.5) );
   ret.b = ( YBit + 2.018*(dstCb - 0.5) );

   ret.rgb = lerp( bg.rgb, clamp( ret.rgb, 0.0, 1.0 ).rgb, fg.a * Opacity );
   ret.a = 1.0;

   return ret;
}
//--------------------------------------------------------------//
float4 Dodge_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   float4 ret = bg / ( 1.0 - fg );
   ret = lerp( bg, ret, fg.a * Opacity );
   ret.a = 1.0;

   return ret;
}

//--------------------------------------------------------------//
float4 Burn_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fg = tex2D( FgSampler, xy1 );
   float4 bg = tex2D( BgSampler, xy2 );

   float4 ret = 1.0 - ( ( 1.0 - bg ) / fg );
   ret = lerp( bg, clamp( ret, 0.0, 1.0 ), fg.a * Opacity );
   ret.a = 1.0;

   return ret;
}

//--------------------------------------------------------------//
// Techniques
//--------------------------------------------------------------//
technique InFront    { pass SinglePass { PixelShader = compile PROFILE InFront_main(); } }
technique Add        { pass SinglePass { PixelShader = compile PROFILE Add_main(); } }
technique Subtract   { pass SinglePass { PixelShader = compile PROFILE Subtract_main(); } }
technique Multiply   { pass SinglePass { PixelShader = compile PROFILE Multiply_main(); } }
technique Screen     { pass SinglePass { PixelShader = compile PROFILE Screen_main(); } }
technique Overlay    { pass SinglePass { PixelShader = compile PROFILE Overlay_main(); } }
technique SoftLight  { pass SinglePass { PixelShader = compile PROFILE SoftLight_main(); } }
technique Hardlight  { pass SinglePass { PixelShader = compile PROFILE Hardlight_main(); } }
technique Exclusion  { pass SinglePass { PixelShader = compile PROFILE Exclusion_main(); } }
technique Lighten    { pass SinglePass { PixelShader = compile PROFILE Lighten_main(); } }
technique Darken     { pass SinglePass { PixelShader = compile PROFILE Darken_main(); } }
technique Average    { pass SinglePass { PixelShader = compile PROFILE Average_main(); } }
technique Difference { pass SinglePass { PixelShader = compile PROFILE Difference_main(); } }
technique Color      { pass SinglePass { PixelShader = compile PROFILE Color_main(); } }
technique Luminosity { pass SinglePass { PixelShader = compile PROFILE Luminosity_main(); } }
technique Dodge      { pass SinglePass { PixelShader = compile PROFILE Dodge_main(); } }
technique Burn       { pass SinglePass { PixelShader = compile PROFILE Burn_main(); } }
