//--------------------------------------------------------------//
// ChromaKey2
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Chromakey";
   string Category    = "Keying";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//

float4 KeyColour
<
   string Description = "Key Colour";
   string Flags = "SpecifiesColourRange";
>;

float4 Tolerance
<
   string Description = "Tolerance";
   string Flags = "SpecifiesColourRange";
   bool Visible = false;
>;

float4 ToleranceSoftness
<
   string Description = "Tolerance softness";
   string Flags = "SpecifiesColourRange";
   bool Visible = false;
>;

float KeySoftAmount
<
   string Description = "Key softness";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.5;

float RemoveSpill
<
   string Description = "Remove spill";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.5;

bool Invert
<
   string Description = "Invert";
> = false;

bool Reveal
<
   string Description = "Reveal";
> = false;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture fg;
texture bg;

//--------------------------------------------------------------//
// Intermediate textures
//--------------------------------------------------------------//
texture RawKey : RenderColorTarget;
texture BlurredKey1 : RenderColorTarget;
texture BlurredKey2 : RenderColorTarget;

//--------------------------------------------------------------//
// Samplers - one for each texture
//--------------------------------------------------------------//
sampler FgSampler          = sampler_state { Texture = <fg>; };
sampler BgSampler          = sampler_state { Texture = <bg>; };

sampler RawKeySampler      = sampler_state
{
   Texture = <RawKey>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;
};
sampler BlurredKey1Sampler = sampler_state
{
   Texture = <BlurredKey1>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;
};
sampler BlurredKey2Sampler = sampler_state
{
   Texture = <BlurredKey2>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;
};

//--------------------------------------------------------------//
// Common
//--------------------------------------------------------------//
const float _FallOff = 0.12;

float _OutputWidth  = 1.0;
float _OutputHeight = 1.0;

#define HUE_IDX 0
#define SAT_IDX 1
#define VAL_IDX 2

const float _oneSixth = 1.0 / 6.0;

const float _minTolerance = 1.0 / 256.0;

bool allPositive( float4 pixel )
{
   return pixel.r > 0 && pixel.g > 0 && pixel.b > 0; 
}

//--------------------------------------------------------------//
// keygen_ps_main
//
// Convert the source to HSV and then compute its similarity with
// the specified key-colour
//--------------------------------------------------------------//
float4 keygen_ps_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float keyVal = 1.0;
   float hueSimilarity = 1.0;

   float4 tolerance1 = Tolerance + _minTolerance;
   float4 tolerance2 = tolerance1 + ToleranceSoftness;

   float4 hsva = 0.0;
   float4 rgba = tex2D( FgSampler, xy1 );

   float maxComponentVal = max( max( rgba.r, rgba.g ), rgba.b );
   float minComponentVal = min( min( rgba.r, rgba.g ), rgba.b );
   float componentRange  = maxComponentVal - minComponentVal;

   hsva[ VAL_IDX ] = maxComponentVal;
   hsva[ SAT_IDX ] = componentRange / maxComponentVal;

   if ( hsva[ SAT_IDX ] == 0 )
   {
      hsva[ HUE_IDX ] = 0;   // undefined
   }
   else
   {
      if ( rgba.r == maxComponentVal )
      {
         hsva[ HUE_IDX ] = ( rgba.g - rgba.b ) / componentRange;
      }
      else if ( rgba.g == maxComponentVal )
      {
         hsva[ HUE_IDX ] = 2 + ( ( rgba.b - rgba.r ) / componentRange );
      }
      else
      {
         hsva[ HUE_IDX ] = 4 + ( ( rgba.r - rgba.g ) / componentRange );
      }

      hsva[ HUE_IDX ] *= _oneSixth;
      if (hsva[ HUE_IDX ] < 0) hsva[ HUE_IDX ] += 1.0;
   }

   // Calc difference between current pixel and specified key-colour
   float4 diff = abs( hsva - KeyColour );

   if ( diff[ HUE_IDX ] > 0.5 )
      diff[ HUE_IDX ] = 1.0 - diff[ HUE_IDX ];

   // Work out how transparent/opaque the corrected pixel will be
   if ( allPositive( tolerance2 - diff ) )
   {
      if ( allPositive( tolerance1 - diff ) )
      {
         keyVal = 0.0;
      }
      else
      {
         diff -= tolerance1;
         hueSimilarity = diff[ HUE_IDX ];
         diff /= ToleranceSoftness;
         keyVal = max( diff[ HUE_IDX ], max( diff[ SAT_IDX ], diff[ VAL_IDX ] ) );
         keyVal = pow( keyVal, 0.25 );
      }
   }
   else
   {
      diff -= tolerance1;
      hueSimilarity = diff[ HUE_IDX ];
   }

   return float4( keyVal, keyVal, keyVal, 1.0 - hueSimilarity );
}

const float blur[] = { 20.0 / 64.0, 15.0 / 64.0, 6.0  / 64.0, 1.0  / 64.0 };  // See Pascals Triangle

//--------------------------------------------------------------//
// Blur1
//--------------------------------------------------------------//
float4 blur1_ps_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float2 onePixAcross   = float2( KeySoftAmount / _OutputWidth, 0.0 );
   float2 twoPixAcross   = onePixAcross * 2.0;
   float2 threePixAcross = onePixAcross * 3.0;

   float4 result = tex2D( RawKeySampler, xy1 );
//return result;

   result.r *= blur[ 0 ];
   result.r += tex2D( RawKeySampler, xy1 + onePixAcross ).r   * blur[ 1 ];
   result.r += tex2D( RawKeySampler, xy1 - onePixAcross ).r   * blur[ 1 ];
   result.r += tex2D( RawKeySampler, xy1 + twoPixAcross ).r   * blur[ 2 ];
   result.r += tex2D( RawKeySampler, xy1 - twoPixAcross ).r   * blur[ 2 ];
   result.r += tex2D( RawKeySampler, xy1 + threePixAcross ).r * blur[ 3 ];
   result.r += tex2D( RawKeySampler, xy1 - threePixAcross ).r * blur[ 3 ];

   return result;
}


//--------------------------------------------------------------//
// Blur2
//--------------------------------------------------------------//
float4 blur2_ps_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float2 onePixDown   = float2( 0.0, KeySoftAmount / _OutputHeight );
   float2 twoPixDown   = onePixDown * 2.0;
   float2 threePixDown = onePixDown * 3.0;

   float4 result = tex2D( BlurredKey1Sampler, xy1 );
//return result;

   result.r *= blur[ 0 ];
   result.r += tex2D( BlurredKey1Sampler, xy1 + onePixDown ).r   * blur[ 1 ];
   result.r += tex2D( BlurredKey1Sampler, xy1 - onePixDown ).r   * blur[ 1 ];
   result.r += tex2D( BlurredKey1Sampler, xy1 + twoPixDown ).r   * blur[ 2 ];
   result.r += tex2D( BlurredKey1Sampler, xy1 - twoPixDown ).r   * blur[ 2 ];
   result.r += tex2D( BlurredKey1Sampler, xy1 + threePixDown ).r * blur[ 3 ];
   result.r += tex2D( BlurredKey1Sampler, xy1 - threePixDown ).r * blur[ 3 ];

   return result;
}

//--------------------------------------------------------------//
// Composite
//
// Blend the foreground with the background using the key that
// was built in keygen_ps_main.  Apply spill-suppression as we go.
//--------------------------------------------------------------//
float4 composite_ps_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 result;

   float4 fg  = tex2D( FgSampler, xy1 );
   float4 bg  = tex2D( BgSampler, xy2 );
   float4 key = tex2D( BlurredKey2Sampler, xy1 );

   // key.r = blurred key
   // key.g = raw, unblurred key
   // key.a = spill removal amount

   // Using min( key.r, key.g ) means that any softness around
   // the key causes the foreground to shrink in from the edges
   float mix = saturate( ( 1.0 - min( key.r, key.g ) * fg.a ) * 2 );

   if ( Reveal )
   {
      result = lerp( mix, 1.0 - mix, Invert );
      result.a = 1.0;
   }
   else if ( Invert )
   {
      result = lerp( bg, fg, mix * bg.a );
      result.a = max( bg.a, mix );
   }
   else
   {
      if ( key.a > 0.8 )
      {
         float4 fgLum = ( fg.r + fg.g + fg.b ) / 3.0;

         // Remove spill..
         fg = lerp( fg, fgLum, ( ( key.a - 0.8 ) / 0.2 ) * RemoveSpill );
      }

      result = lerp( fg, bg, mix * bg.a );
      result.a = max( bg.a, 1.0 - mix );
   }

   return result;
}

//--------------------------------------------------------------//
technique ChromaKey
{
   pass KeyGen
   <
      string Script = "RenderColorTarget0 = RawKey;";
   >
   {
      PixelShader = compile PROFILE keygen_ps_main();
   }

   pass Blur1
   <
      string Script = "RenderColorTarget0 = BlurredKey1;";
   >
   {
      PixelShader = compile PROFILE blur1_ps_main();
   }

   pass Blur2
   <
      string Script = "RenderColorTarget0 = BlurredKey2;";
   >
   {
      PixelShader = compile PROFILE blur2_ps_main();
   }

   pass Composite
   {
      PixelShader = compile PROFILE composite_ps_main();
   }
}
