//--------------------------------------------------------------//
// DVE1
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "DVE1";
   string Category    = "Null";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
float CentreX
<
   string Description = "Position";
   string Flags = "SpecifiesPointX";
   float MinVal = -1.00;
   float MaxVal = 2.00;
> = 0.5;

float CentreY
<
   string Description = "Position";
   string Flags = "SpecifiesPointY";
   float MinVal = -1.00;
   float MaxVal = 2.00;
> = 0.5;

float XScale
<
   string Description = "X Scale";
   string Group = "Scale";
   float MinVal = 0.00;
   float MaxVal = 10.00;
> = 1.0;

float YScale
<
   string Description = "Y Scale";
   string Group = "Scale";
   float MinVal = 0.00;
   float MaxVal = 10.00;
> = 1.0;

float CropLeft
<
   string Description = "Left";
   string Group = "Crop";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.0;

float CropBottom
<
   string Description = "Bottom";
   string Group = "Crop";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.0;

float CropRight
<
   string Description = "Right";
   string Group = "Crop";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.0;

float CropTop
<
   string Description = "Top";
   string Group = "Crop";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture Input;
texture _lut;

sampler InputSampler = sampler_state
{
   Texture = <Input>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;
   ADDRESSU  = ClampToEdge;
   ADDRESSV  = ClampToEdge;
};

sampler3D CubeSampler = sampler_state
{
  Texture   = <_lut>;
  MagFilter = Linear;
  MinFilter = Linear;
};

float _scale  = 1.0f;
float _offset = 0.0f;

float _InputNormWidth = 1.0;

static const float4 _opaqueBlack = float4( 0,0,0,1 );

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 scaleCrop( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 ret;

   float x        = CentreX;
   float y        = 1.0 - CentreY;
   float l        = x - ( XScale / 2.0 );
   float t        = y - ( YScale / 2.0 );
   float r        = l + XScale;
   float b        = t + YScale;
   float croppedL = l + ( CropLeft * XScale );
   float croppedT = t + ( CropTop  * YScale );
   float croppedR = r - ( CropRight * XScale );
   float croppedB = b - ( CropBottom * YScale );

   if ( xy0.x < croppedL ||  xy0.x > croppedR || xy0.y < croppedT || xy0.y > croppedB )
   {
      ret = _opaqueBlack;
   }
   else
   {
      float2 fgPos = float2( ( xy1.x - l ) / XScale, ( xy1.y - t ) / YScale );

      ret = tex2D( InputSampler, fgPos );
   }

   return ret;
}
//--------------------------------------------------------------//
float4 scaleCropLUT( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 source;

   float x        = CentreX;
   float y        = 1.0 - CentreY;
   float l        = x - ( XScale / 2.0 );
   float t        = y - ( YScale / 2.0 );
   float r        = l + XScale;
   float b        = t + YScale;
   float croppedL = l + ( CropLeft * XScale );
   float croppedT = t + ( CropTop  * YScale );
   float croppedR = r - ( CropRight * XScale );
   float croppedB = b - ( CropBottom * YScale );

   if ( xy0.x < croppedL ||  xy0.x > croppedR || xy0.y < croppedT || xy0.y > croppedB )
   {
      source = _opaqueBlack;
   }
   else
   {
      float2 fgPos = float2( ( xy1.x - l ) / XScale, ( xy1.y - t ) / YScale );

      source = tex2D( InputSampler, fgPos );
   }

   float4 ret = tex3D( CubeSampler, ( source.rgb * _scale ) + _offset );
   ret.a = source.a;

   return ret;
}

//--------------------------------------------------------------//
float4 LUTOnly( float2 fgPos : TEXCOORD1 ) : COLOR
{
   float4 source = tex2D( InputSampler, fgPos );
   float4 ret = tex3D( CubeSampler, ( source.rgb * _scale ) + _offset );
   ret.a = source.a;

   return ret;
}

//--------------------------------------------------------------//
// Techniques
//--------------------------------------------------------------//
technique ScaleCrop    { pass p1 { PixelShader = compile PROFILE scaleCrop(); } }
technique ScaleCropLUT { pass p1 { PixelShader = compile PROFILE scaleCropLUT(); } }
technique LUT          { pass p1 { PixelShader = compile PROFILE LUTOnly(); } }
