//--------------------------------------------------------------//
// Glow
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Glow";
   string Category    = "Stylize";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
int SetTechnique
<
   string Description = "Source";
   string Enum = "Luminance,Red,Green,Blue";
> = 0;

float Tolerance
<
   string Description = "Tolerance";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.5;

float Feather
<
   string Description = "Feather";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.0;

float Size
<
   string Description = "Size";
   float MinVal = 1.00;
   float MaxVal = 10.00;
> = 4.0;

float Strength
<
   string Description = "Strength";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.5;

float4 Colour
<
   string Description = "Colour";
> = { 1.0, 1.0, 1.0, 1.0 };

float _OutputWidth  = 1.0;
float _OutputHeight = 1.0;
const float _MaxFeather = 0.5;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture Input;
sampler InputSampler = sampler_state
{
   Texture = <Input>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;

};

texture Glow1 : RenderColorTarget;
sampler GlowSampler = sampler_state
{
   Texture = <Glow1>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;
};

texture Glow2 : RenderColorTarget;
sampler Glow2Sampler = sampler_state
{
   Texture = <Glow2>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;
};

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 lum_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float actualFeather = Feather * _MaxFeather;

   float4 ret = tex2D( InputSampler, xy1 );

   float srcLum = ( ( ret.r * 0.3 ) + ( ret.g * 0.59 ) + ( ret.b * 0.11 ) ) * ret.a;

   if ( srcLum < Tolerance )
   {
      ret = 0;
   }
   else if ( srcLum < ( Tolerance + actualFeather ) )
   {
      ret = lerp( 0, Colour, ( srcLum - Tolerance ) / actualFeather  );
   }
   else
   {
      ret = Colour;
   }

   return ret;
}
//--------------------------------------------------------------//
float4 red_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 ret = tex2D( InputSampler, xy1 );

   if ( ( ret.r * ret.a ) < Tolerance )
   {
      ret = 0;
   }

   return ret;
}
//--------------------------------------------------------------//
float4 green_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 ret = tex2D( InputSampler, xy1 );

   if ( ( ret.g * ret.a ) < Tolerance )
   {
      ret = 0;
   }

   return ret;
}
//--------------------------------------------------------------//
float4 blue_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 ret = tex2D( InputSampler, xy1 );

   if ( ( ret.b * ret.a ) < Tolerance )
   {
      ret = 0;
   }

   return ret;
}
//--------------------------------------------------------------//
float4 glowX_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float one   = Size / _OutputWidth;
   float tap1  = xy1.x + one;
   float tap2  = tap1 + one;
   float tap3  = tap2 + one;;
   float tap4  = tap3 + one;
   float tap5  = tap4 + one;
   float tap6  = tap5 + one;
   float ntap1 = xy1.x - one;
   float ntap2 = ntap1 - one;
   float ntap3 = ntap2 - one;;
   float ntap4 = ntap3 - one;
   float ntap5 = ntap4 - one;
   float ntap6 = ntap5 - one;

   float4 blurred = tex2D( GlowSampler, xy1 );
   blurred += tex2D( GlowSampler, float2( tap1,  xy1.y ) );
   blurred += tex2D( GlowSampler, float2( tap2,  xy1.y ) );
   blurred += tex2D( GlowSampler, float2( tap3,  xy1.y ) );
   blurred += tex2D( GlowSampler, float2( tap4,  xy1.y ) );
   blurred += tex2D( GlowSampler, float2( tap5,  xy1.y ) );
   blurred += tex2D( GlowSampler, float2( tap6,  xy1.y ) );
   blurred += tex2D( GlowSampler, float2( ntap1, xy1.y ) );
   blurred += tex2D( GlowSampler, float2( ntap2, xy1.y ) );
   blurred += tex2D( GlowSampler, float2( ntap3, xy1.y ) );
   blurred += tex2D( GlowSampler, float2( ntap4, xy1.y ) );
   blurred += tex2D( GlowSampler, float2( ntap5, xy1.y ) );
   blurred += tex2D( GlowSampler, float2( ntap6, xy1.y ) );

   return blurred / 13;
}

//--------------------------------------------------------------//
float4 glowY_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float one  = Size / _OutputHeight;
   float tap1 = xy1.y + one;
   float tap2 = tap1 + one;
   float tap3 = tap2 + one;;
   float tap4 = tap3 + one;
   float tap5 = tap4 + one;
   float tap6 = tap5 + one;

   float ntap1 = xy1.y - one;
   float ntap2 = ntap1 - one;
   float ntap3 = ntap2 - one;;
   float ntap4 = ntap3 - one;
   float ntap5 = ntap4 - one;
   float ntap6 = ntap5 - one;

   float4 blurred = tex2D( Glow2Sampler, xy1 );
   blurred += tex2D( Glow2Sampler, float2( xy1.x, tap1 ) );
   blurred += tex2D( Glow2Sampler, float2( xy1.x, tap2 ) );
   blurred += tex2D( Glow2Sampler, float2( xy1.x, tap3 ) );
   blurred += tex2D( Glow2Sampler, float2( xy1.x, tap4 ) );
   blurred += tex2D( Glow2Sampler, float2( xy1.x, tap5 ) );
   blurred += tex2D( Glow2Sampler, float2( xy1.x, tap6 ) );
   blurred += tex2D( Glow2Sampler, float2( xy1.x, ntap1 ) );
   blurred += tex2D( Glow2Sampler, float2( xy1.x, ntap2 ) );
   blurred += tex2D( Glow2Sampler, float2( xy1.x, ntap3 ) );
   blurred += tex2D( Glow2Sampler, float2( xy1.x, ntap4 ) );
   blurred += tex2D( Glow2Sampler, float2( xy1.x, ntap5 ) );
   blurred += tex2D( Glow2Sampler, float2( xy1.x, ntap6 ) );

   float4 source = tex2D( InputSampler, xy1 );
   float4 ret    = lerp( source, source + ( blurred / 13 ), Strength );
   ret.a = source.a;

   return ret;
}
//--------------------------------------------------------------//
technique LuminanceGlow
{
   pass Composite
   <
      string Script = "RenderColorTarget0 = Glow1;";
   >
   {
      PixelShader = compile PROFILE lum_main();
   }

   pass BlurX
   <
      string Script = "RenderColorTarget0 = Glow2;";
   >
   {
      PixelShader = compile PROFILE glowX_main();
   }

   pass BlurY
   {
      PixelShader = compile PROFILE glowY_main();
   }
}
//--------------------------------------------------------------//
technique RedGlow
{
   pass Composite
   <
      string Script = "RenderColorTarget0 = Glow1;";
   >
   {
      PixelShader = compile PROFILE red_main();
   }

   pass BlurX
   <
      string Script = "RenderColorTarget0 = Glow2;";
   >
   {
      PixelShader = compile PROFILE glowX_main();
   }

   pass BlurY
   {
      PixelShader = compile PROFILE glowY_main();
   }
}

//--------------------------------------------------------------//
technique GreenGlow
{
   pass Composite
   <
      string Script = "RenderColorTarget0 = Glow1;";
   >
   {
      PixelShader = compile PROFILE green_main();
   }

   pass BlurX
   <
      string Script = "RenderColorTarget0 = Glow2;";
   >
   {
      PixelShader = compile PROFILE glowX_main();
   }

   pass BlurY
   {
      PixelShader = compile PROFILE glowY_main();
   }
}
//--------------------------------------------------------------//
technique BlueGlow
{
   pass Composite
   <
      string Script = "RenderColorTarget0 = Glow1;";
   >
   {
      PixelShader = compile PROFILE blue_main();
   }

   pass BlurX
   <
      string Script = "RenderColorTarget0 = Glow2;";
   >
   {
      PixelShader = compile PROFILE glowX_main();
   }

   pass BlurY
   {
      PixelShader = compile PROFILE glowY_main();
   }
}

