//--------------------------------------------------------------//
// Gradient Colour Effects
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Colour Gradient";
   string Category    = "Mattes";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//

int SetTechnique
<
   string Description = "Type";
   string Enum = "Top to bottom,Left to right,Radial";
> = 0;

float4 ColourOne
<
   string Description = "Colour";
   string Group = "Colour One";
   bool SupportsAlpha = true;
> = { 1.0, 0.0, 0.0, 1.0 };

float ColourOneExtent
<
   string Description = "Extent";
   string Group = "Colour One";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 1.00;

float4 ColourTwo
<
   string Description = "Colour";
   string Group = "Colour Two";
   bool SupportsAlpha = true;
> = { 1.0, 1.0, 0.0, 1.0 };

float ColourTwoExtent
<
   string Description = "Extent";
   string Group = "Colour Two";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 1.00;

texture Input;
sampler2D InputSampler = sampler_state { Texture = <Input>; };


//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 topToBottom_main( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1 ) : COLOR
{
   float progress = xyNorm.y;
   float4 pixel = tex2D( InputSampler, xy1 );

   float colOneExtent = max( ColourOneExtent, 0.01 );
   float colTwoExtent = max( ColourTwoExtent, 0.01 );

   float colOneAmt = max( ColourOneExtent - progress, 0.0 ) / colOneExtent;
   float colTwoAmt = max( ColourTwoExtent - ( 1.0 - progress ), 0.0 ) / colTwoExtent;

   float4 ret  = ( lerp( pixel, ColourOne, ColourOne.a ) * colOneAmt ) +
                 ( lerp( pixel, ColourTwo, ColourTwo.a ) * colTwoAmt ) +
                 ( pixel * ( 1.0 - ( colOneAmt + colTwoAmt ) ) );

   ret.a = pixel.a;

   return ret;
}

float4 leftToRight_main( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1 ) : COLOR
{
   float progress = xyNorm.x;
   float4 pixel = tex2D( InputSampler, xy1 );

   float colOneExtent = max( ColourOneExtent, 0.01 );
   float colTwoExtent = max( ColourTwoExtent, 0.01 );

   float colOneAmt = max( ColourOneExtent - progress, 0.0 ) / colOneExtent;
   float colTwoAmt = max( ColourTwoExtent - ( 1.0 - progress ), 0.0 ) / colTwoExtent;

   float4 ret  = ( lerp( pixel, ColourOne, ColourOne.a ) * colOneAmt ) +
                 ( lerp( pixel, ColourTwo, ColourTwo.a ) * colTwoAmt ) +
                 ( pixel * ( 1.0 - ( colOneAmt + colTwoAmt ) ) );

   ret.a = pixel.a;

   return ret;
}

float4 radial_main( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1 ) : COLOR
{
   float progress = abs( distance( xyNorm, float2( 0.5, 0.5 ) ) ) * 1.414; // Sqrt(2)
   float4 pixel = tex2D( InputSampler, xy1 );

   float colOneExtent = max( ColourOneExtent, 0.01 );
   float colTwoExtent = max( ColourTwoExtent, 0.01 );

   float colOneAmt = max( ColourOneExtent - progress, 0.0 ) / colOneExtent;
   float colTwoAmt = max( ColourTwoExtent - ( 1.0 - progress ), 0.0 ) / colTwoExtent;

   float4 ret  = ( lerp( pixel, ColourOne, ColourOne.a ) * colOneAmt ) +
                 ( lerp( pixel, ColourTwo, ColourTwo.a ) * colTwoAmt ) +
                 ( pixel * ( 1.0 - ( colOneAmt + colTwoAmt ) ) );

   ret.a = pixel.a;

   return ret;
}

//--------------------------------------------------------------//
// Techniques
//--------------------------------------------------------------//
technique TopToBottom { pass SinglePass { PixelShader = compile PROFILE topToBottom_main(); } }
technique LeftToRight { pass SinglePass { PixelShader = compile PROFILE leftToRight_main(); } }
technique Radial      { pass SinglePass { PixelShader = compile PROFILE radial_main(); } }
