//--------------------------------------------------------------//
// Gradient Colour Effects
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Corner Gradient";
   string Category    = "Mattes";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//

float4 TLColour
<
   string Description = "Top Left";
   string Group = "Colours";
   bool SupportsAlpha = true;
> = { 1.0, 1.0, 0.0, 1.0 };

float4 TRColour
<
   string Description = "Top Right";
   string Group = "Colours";
   bool SupportsAlpha = true;
> = { 1.0, 0.0, 0.0, 1.0 };

float4 BLColour
<
   string Description = "Bottom Left";
   string Group = "Colours";
   bool SupportsAlpha = true;
> = { 0.0, 0.0, 1.0, 1.0 };

float4 BRColour
<
   string Description = "Bottom Right";
   string Group = "Colours";
   bool SupportsAlpha = true;
> = { 0.0, 1.0, 1.0, 1.0 };

float Left
<
   string Description = "Left";
   string Group = "Bounds";
   string Flags = "SpecifiesBoxL"; // Subsequent related params must be in the order B,R,T
   float MinVal = 0.0;
   float MaxVal = 1.0;
> = 0.0;

float Bottom
<
   string Description = "Bottom";
   string Group = "Bounds";
   string Flags = "SpecifiesBoxB";
   float MinVal = 0.0;
   float MaxVal = 1.0;
> = 0.0;

float Right
<
   string Description = "Right";
   string Group = "Bounds";
   string Flags = "SpecifiesBoxR";
   float MinVal = 0.0;
   float MaxVal = 1.0;
> = 1.0;

float Top
<
   string Description = "Top";
   string Group = "Bounds";
   string Flags = "SpecifiesBoxT";
   float MinVal = 0.0;
   float MaxVal = 1.0;
> = 1.0;

texture Input;
sampler InputSampler = sampler_state { Texture = <Input>; };


//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 ps_main( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 ret;
   float4 pixel = tex2D( InputSampler, xy1 );

   float top = 1.0 - Top;
   float bottom = 1.0 - Bottom;

   if ( xyNorm.x >= Left && xyNorm.x <= Right && xyNorm.y >= top && xyNorm.y <= bottom )
   {
      float4 tmp1  = lerp( pixel, TLColour, TLColour.a );
      float4 tmp2  = lerp( pixel, TRColour, TRColour.a );
      float4 tmp3  = lerp( pixel, BLColour, BLColour.a );
      float4 tmp4  = lerp( pixel, BRColour, BRColour.a );

      float xAmount = ( xyNorm.x - Left ) / ( Right - Left );
      float yAmount = ( xyNorm.y - top ) / ( bottom - top );

      float4 tmp5  = lerp( tmp1, tmp2, xAmount );
      float4 tmp6  = lerp( tmp3, tmp4, xAmount );

      ret   = lerp( tmp5, tmp6, yAmount );
      ret.a = pixel.a;
   }
   else
      ret = pixel;

   return ret;
}

//--------------------------------------------------------------//
// Technique Section for Image Processing Effects
//--------------------------------------------------------------//
technique CornerGradient { pass Single_Pass {PixelShader = compile PROFILE ps_main(); } }
