//--------------------------------------------------------------//
// Lumakey
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Lumakey";
   string Category    = "Keying";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
float Tolerance
<
   string Description = "Tolerance";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.5;

float EdgeSoftness
<
   string Description = "Edge Softness";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.1;

bool Invert
<
   string Description = "Invert";
> = false ;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture Fg;
sampler FgSampler = sampler_state { Texture = <Fg>; };

texture Bg;
sampler2D BgSampler = sampler_state { Texture = <Bg>; };

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 ps_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 ret;
   float4 src1 = tex2D( FgSampler, xy1 );
   float4 src2 = tex2D( BgSampler, xy2 );

   float srcLum = ( ( src1.r * 0.3 ) + ( src1.g * 0.59 ) + ( src1.b * 0.11 ) );

   if ( Invert )
   {
      float softStart = Tolerance;
      float softEnd   = softStart + EdgeSoftness;

      if ( srcLum > softEnd )
      {
         ret = src2;
      }
      else
      {
         if ( srcLum > softStart )
         {
            ret = lerp( src1, src2, ( srcLum - softStart ) / EdgeSoftness );
         }
         else
         {
            ret = src1;
         }
      }
   }
   else
   {
      float softStart = 1.0 - Tolerance;
      float softEnd   = softStart + EdgeSoftness;

      if ( srcLum > softEnd )
      {
         ret = src1;
      }
      else
      {
         if ( srcLum > softStart )
         {
            ret = lerp( src2, src1, ( srcLum - softStart ) / EdgeSoftness );
         }
         else
         {
            ret = src2;
         }
      }
   }

   return ret;
}

//--------------------------------------------------------------//
// Techniques
//--------------------------------------------------------------//
technique Lumakey { pass Single_Pass { PixelShader = compile PROFILE ps_main(); } }
