//--------------------------------------------------------------//
// LumaWipe
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Luma Wipe";
   string Category    = "Mixes";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
int SetTechnique
<
   string Description = "Luma source";
   string Enum = "Foreground,Background";
> = 0;

float BorderWidth
<
   string Description = "Size";
   string Group       = "Border";
   float MinVal = 0.0;
   float MaxVal = 1.0;
> = 0.0;

float4 BorderColour1
<
   string Description = "First colour";
   string Group       = "Border";
> = { 1.0, 1.0, 1.0, 1.0 };

float4 BorderColour2
<
   string Description = "Second colour";
   string Group       = "Border";
> = { 1.0, 1.0, 1.0, 1.0 };


float Softness
<
   string Description = "Softness";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.2;

float Progress
<
   string Description = "Progress";
   float MinVal = 0.0;
   float MaxVal = 1.0;
   float KF0    = 0.0;
   float KF1    = 1.0;
> = 0.0;

bool Invert
<
   string Description = "Invert";
> = false ;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture fg;
texture bg;

//--------------------------------------------------------------//
// Samplers
//--------------------------------------------------------------//
sampler FgSampler = sampler_state { Texture = <fg>; };
sampler BgSampler = sampler_state { Texture = <bg>; };

//-----------------------------------------------------------------------------------------
float4 doWipe( float4 fgPix, float4 bgPix, float luma )
{
   float totalEdgeSize = Softness + BorderWidth;
   float halfSoftness  = ( Softness / 2.0 );
   float edgeSoftStart = ( Progress * ( 1.0 + totalEdgeSize ) ) - totalEdgeSize;
   float borderStart   = edgeSoftStart + halfSoftness;
   float borderEnd     = borderStart + BorderWidth;
   float edgeSoftEnd   = borderEnd + halfSoftness;

   float4 ret;

   luma = lerp( luma, 1.0 - luma, Invert );

   if ( luma > edgeSoftEnd )
   {
      ret = fgPix;
   }
   else if ( luma < edgeSoftStart )
   {
      ret = bgPix;
   }
   else
   {
      float lumaMinusEdgeStart =  luma - edgeSoftStart;

      if ( BorderWidth == 0 )
      {
         ret = lerp( bgPix, fgPix, lumaMinusEdgeStart / Softness );
      }
      else
      {
         float4 borderColour = lerp( BorderColour1, BorderColour2, lumaMinusEdgeStart / totalEdgeSize );

         if ( luma < borderStart )
            ret = lerp( bgPix, borderColour, lumaMinusEdgeStart / halfSoftness );
         else if ( luma > borderEnd )
            ret = lerp( fgPix, borderColour, ( edgeSoftEnd - luma ) / halfSoftness );
         else
            ret = borderColour;
      }
   }

   return ret;
}


//--------------------------------------------------------------//
// Pixel Shader
//--------------------------------------------------------------//
float4 fg_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fgPix = tex2D( FgSampler, xy1 );
   float4 bgPix = tex2D( BgSampler, xy2 );

   return doWipe( fgPix, bgPix, ( fgPix.r + fgPix.g + fgPix.b ) / 3.0 );
}
//--------------------------------------------------------------//
float4 bg_main( float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 fgPix = tex2D( FgSampler, xy1 );
   float4 bgPix = tex2D( BgSampler, xy2 );

   return doWipe( fgPix, bgPix, ( bgPix.r + bgPix.g + bgPix.b ) / 3.0 );
}

//--------------------------------------------------------------//
// Techniques
//--------------------------------------------------------------//
technique Foreground { pass PassOne { PixelShader = compile PROFILE fg_main(); } }
technique Background { pass PassOne { PixelShader = compile PROFILE bg_main(); } }
