//--------------------------------------------------------------//
// Negative
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Negative";
   string Category    = "Colour Effects";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
float Level
<
   string Description = "Level";
   string Group = "Threshold";
   float MinVal = 0.0;
   float MaxVal = 1.0;
> = 0.0;

float Softness
<
   string Description = "Softness";
   string Group = "Threshold";
   float MinVal = 0.0;
   float MaxVal = 1.0;
> = 0.25;

float _MaxSoftness = 0.2;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture Input;
sampler InputSampler = sampler_state { Texture = <Input>; };

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 ps_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 ret;

   float soft = Softness * _MaxSoftness;
   float softStart = Level - soft;

   // Get source pixel..
   float4 srcPixel = tex2D( InputSampler, xy1 );

   float lum = ( srcPixel.r + srcPixel.g + srcPixel.b ) / 3.0;

   if ( lum >= Level )
   {
      // Calculate the inverse
      ret = 1.0 - srcPixel;
   }
   else if ( lum > softStart )
   {
      ret = lerp( srcPixel, 1.0 - srcPixel, ( lum - softStart ) / soft );
   }
   else
   {
      ret = srcPixel;
   }

   ret = lerp( srcPixel, ret, srcPixel.a );
   ret.a = srcPixel.a;

   return ret;
}

//--------------------------------------------------------------//
// Technique Section for Image Processing Effects
//--------------------------------------------------------------//
technique Negative { pass Single_Pass { PixelShader = compile PROFILE ps_main(); } }
