//--------------------------------------------------------------//
// Null
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Null";
   string Category    = "Null";
> = 0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture Input;
sampler InputSampler = sampler_state { Texture = <Input>; };
sampler InputSamplerPoint = sampler_state
{
   Texture = <Input>;
   MINFILTER = POINT;
   MIPFILTER = POINT;
   MAGFILTER = POINT;
};

float Opacity
<
   string Description = "Opacity";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 1.0;

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 null_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   return tex2D( InputSampler, xy1 );
}

float4 revComp_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );

   float4 ret;
   ret.a = src.b;
   ret.r = src.g;
   ret.g = src.r;
   ret.b = src.a;

   return ret;
}

float4 flip_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   return tex2D( InputSampler, float2( xy1.x, 1.0 - xy1.y ) );
}

float4 revAndFlip_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, float2( xy1.x, 1.0 - xy1.y ) );

   float4 ret;
   ret.r = src.a;
   ret.g = src.b;
   ret.b = src.g;
   ret.a = src.r;

   return ret;
}

float4 UYVYtoYUYV_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );

   float4 ret;
   ret.r = src.a;
   ret.g = src.b;
   ret.b = src.g;
   ret.a = src.r;

   return ret;
}

float4 YVYUtoYUYV_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );

   float4 ret;
   ret.r = src.r;
   ret.g = src.a;
   ret.b = src.b;
   ret.a = src.g;

   return ret;
}

bool isEven( float f )
{
  return ( ( 2 * floor( f / 2 ) ) == f );
}

float _Width = 1280;
float _Height;

float4 YV12toYUYV_main( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1 ) : COLOR
{
   float x = floor( ( xyNorm.x * _Width ) + 0.5 );
   float y = floor( ( xyNorm.y * _Height ) + 0.5 );

   bool xEven = isEven( x );
   bool yEven = isEven( y );

   float yX = floor( x / 2 );

   if ( !yEven )
      yX += _Width/2;

   yX /= _Width;

   float uvX = ( xyNorm.x / 4 ) + ( fmod( y,4 ) * 0.25 );

   float4 yPixel = tex2D( InputSampler, float2( yX, xy1.y / 2 ) );
   float4 uPixel = tex2D( InputSampler, float2( uvX, 0.5  + ( xy1.y / 4 ) ) );
   float4 vPixel = tex2D( InputSampler, float2( uvX, 0.75 + ( xy1.y / 4 ) ) );

   float4 yuyv;

   if ( xEven )
   {
      yuyv[0] = yPixel[1];
      yuyv[2] = yPixel[2];
   }
   else
   {
      yuyv[0] = yPixel[3];
      yuyv[2] = yPixel[0];
   }

   int uvIdx = fmod( x, 4 );
   yuyv[1] = uPixel[ uvIdx ];
   yuyv[3] = vPixel[ uvIdx ];

   return yuyv;
}


float4 V210toYUYV_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );

   float4 ret;
   ret.r = src.a;
   ret.g = src.b;
   ret.b = src.g;
   ret.a = src.r;

   return ret;
}

float4 RotateLeft_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );

   float4 ret;
   ret.r = src.a;
   ret.g = src.r;
   ret.b = src.g;
   ret.a = src.b;

   return ret;
}

float4 RotateRight_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );

   float4 ret;
   ret.r = src.g;
   ret.g = src.b;
   ret.b = src.a;
   ret.a = src.r;

   return ret;
}

float4 BGRtoARGB_main( float2 xy : TEXCOORD1, float2 xyl : TEXCOORD2 ) : COLOR
{
   float4 ret;

   float2 xyr;

   // shift a quarter pixel right
   xyr.x = xy.x + 0.25 / _Width;
   xyr.y = xy.y;

   int x = ( int ) fmod( xy.x * _Width, 4 );

   if ( x == 0 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.r;
      ret.g = src.g;
      ret.r = src.b;
   }
   else if ( x == 1 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcr.g;
      ret.g = srcr.b;
      ret.r = srcl.a;
   }
   else if ( x == 2 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcr.b;
      ret.g = srcl.a;
      ret.r = srcl.r;
   }
   else if ( x == 3 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.a;
      ret.g = src.r;
      ret.r = src.g;
   }

   ret.a = 1.;

   return ret;
}

float4 RGBtoARGB_main( float2 xy : TEXCOORD1, float2 xyl : TEXCOORD2 ) : COLOR
{
   float4 ret;

   float2 xyr;

   // shift a quarter pixel right
   xyr.x = xy.x + 0.25 / _Width;
   xyr.y = xy.y;

   int x = ( int ) fmod( xy.x * _Width, 4 );

   if ( x == 0 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.b;
      ret.g = src.g;
      ret.r = src.r;
   }
   else if ( x == 1 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcl.a;
      ret.g = srcr.b;
      ret.r = srcr.g;
   }
   else if ( x == 2 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcl.r;
      ret.g = srcl.a;
      ret.r = srcr.b;
   }
   else if ( x == 3 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.g;
      ret.g = src.r;
      ret.r = src.a;
   }

   ret.a = 1.;

   return ret;
}

float4 RGB16toABGR_main( float2 xy : TEXCOORD1, float2 xyl : TEXCOORD2 ) : COLOR
{
   float4 ret;

   float2 xyr;

   // shift a quarter pixel right
   xyr.x = xy.x + 0.25 / _Width;
   xyr.y = xy.y;

   int x = (int) fmod( xy.x * _Width, 4 );

   if ( x == 0 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.r = src.b;
      ret.g = src.g;
      ret.b = src.r;
   }
   else if ( x == 1 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.r = srcr.g;
      ret.g = srcr.r;
      ret.b = srcl.a;
   }
   else if ( x == 2 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.r = srcr.r;
      ret.g = srcl.a;
      ret.b = srcl.b;
   }
   else if ( x == 3 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.a;
      ret.g = src.b;
      ret.r = src.g;
   }

   ret.a = 1.;

   return ret;
}

// and the reverse? (both surfaces are ARGB but the output wraps RGB)
float4 ARGBtoBGR_main( float2 xy : TEXCOORD1, float2 xyr : TEXCOORD2 ) : COLOR
{
   float4 ret;

   // which group?
   int x = (int) (fmod( xy.x * _Width, 3.0 ));

   if ( x == 0 )
   {
      float4 src = tex2D( InputSamplerPoint, xy );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = src.b;
      ret.g = src.g;
      ret.r = src.r;
      ret.a = srcr.b;
   }
   else if ( x == 1 )
   {
      // 1/3 through a pixel
      //xy.x -= 1.0 / (3.0 * _Width);
      //xyr.x -= 1.0 / (3.0 * _Width);

      float4 src = tex2D( InputSamplerPoint, xy );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = src.g;
      ret.g = src.r;
      ret.r = srcr.b;
      ret.a = srcr.g;
   }
   else if ( x == 2 )
   {
      // 2/3 of the way through the pixel - we would sample the next 2 pixels instead (or suchlike)
      xy.x -= 2.0 / (3.0 * _Width);
      xyr.x -= 2.0 / (3.0 * _Width);

      float4 src = tex2D( InputSamplerPoint, xy );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = src.r;
      ret.g = srcr.b;
      ret.r = srcr.g;
      ret.a = srcr.r;
   }

   return ret;
}

float4 ARGBtoBGRA_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );

   float4 ret = src;
   ret.r = src.a;
   ret.g = src.r;
   ret.b = src.g;
   ret.a = src.b;
   return ret;
}

// the reverse
float4 BGRAtoARGB_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );

   float4 ret = src;
   ret.a = src.r;
   ret.r = src.g;
   ret.g = src.b;
   ret.b = src.a;
   return ret;
}
// swap R and B
float4 ABGRtoARGB_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSampler, xy1 );

   float4 ret = src;
   ret.a = src.a;
   ret.r = src.b;
   ret.g = src.g;
   ret.b = src.r;
   return ret;
}


// and the reverse? (both surfaces are ARGB but the output wraps RGB)
float4 ARGBtoRGB_main( float2 xy : TEXCOORD1, float2 xyr : TEXCOORD2 ) : COLOR
{
   float4 ret;

   // which group?
   int x = (int) (fmod( xy.x * _Width, 3.0 ));

   if ( x == 0 )
   {
      float4 src = tex2D( InputSamplerPoint, xy );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.r = src.r;
      ret.g = src.g;
      ret.b = src.b;
      ret.a = srcr.r;
   }
   else if ( x == 1 )
   {
      // 1/3 through a pixel
      //xy.x -= 1.0 / (3.0 * _Width);
      //xyr.x -= 1.0 / (3.0 * _Width);

      float4 src = tex2D( InputSamplerPoint, xy );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.r = src.g;
      ret.g = src.b;
      ret.b = srcr.r;
      ret.a = srcr.g;
   }
   else if ( x == 2 )
   {
      // 2/3 of the way through the pixel - we would sample the next 2 pixels instead (or suchlike)
      xy.x -= 2.0 / (3.0 * _Width);
      xyr.x -= 2.0 / (3.0 * _Width);

      float4 src = tex2D( InputSamplerPoint, xy );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = src.b;
      ret.g = srcr.r;
      ret.r = srcr.g;
      ret.a = srcr.b;
   }

   return ret;
}

float4 RGBtoARGBFlip_main( float2 xy : TEXCOORD1, float2 xyl : TEXCOORD2 ) : COLOR
{
   float4 ret;

   float2 xyr;

   // shift a quarter pixel right
   xyr.x = xy.x + 0.25 / _Width;

   // flip
   xyr.y = 1 - xy.y;
   xyl.y = 1 - xyl.y;

   int x = ( int ) fmod( xy.x * _Width, 4 );

   if ( x == 0 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.b;
      ret.g = src.g;
      ret.r = src.r;
   }
   else if ( x == 1 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcl.a;
      ret.g = srcr.b;
      ret.r = srcr.g;
   }
   else if ( x == 2 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcl.r;
      ret.g = srcl.a;
      ret.r = srcr.b;
   }
   else if ( x == 3 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.g;
      ret.g = src.r;
      ret.r = src.a;
   }

   ret.a = 1.;

   return ret;
}
float4 RGBPlanartoARGB_main( float2 xy : TEXCOORD1, float2 xyl : TEXCOORD2 ) : COLOR
{
   float4 ret;

   float2 xyr;

   // shift a quarter pixel right
   xyr.x = xy.x + 0.25 / _Width;
   xyr.y = xy.y;

   int x = ( int ) fmod( xy.x * _Width, 4 );

   if ( x == 0 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.b;
      ret.g = src.g;
      ret.r = src.r;
   }
   else if ( x == 1 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcl.a;
      ret.g = srcr.b;
      ret.r = srcr.g;
   }
   else if ( x == 2 )
   {
      float4 srcl = tex2D( InputSamplerPoint, xyl );
      float4 srcr = tex2D( InputSamplerPoint, xyr );

      ret.b = srcl.r;
      ret.g = srcl.a;
      ret.r = srcr.b;
   }
   else if ( x == 3 )
   {
      float4 src = tex2D( InputSamplerPoint, xyr );

      ret.b = src.g;
      ret.g = src.r;
      ret.r = src.a;
   }

   ret.a = 1.;

   return ret;
}

float4 ChannelExpand_main( float2 xy : TEXCOORD1 ) : COLOR
{
   float4 src = tex2D( InputSamplerPoint, xy );

   float f = floor( ( frac( ( xy.x * _Width ) / 4.0 ) * 4.0 ) + 0.5 );

   float4 ret = 0;

   if ( f == 0 )
      ret = src.b;
   else if ( f == 1 )
      ret = src.g;
   else if ( f == 2 )
      ret = src.r;
   else
      ret = src.a;

   ret.a = 1.0;

   return ret;
}

float4 blend_main( float2 xy : TEXCOORD1 ) : COLOR
{
   float4 pixel = tex2D( InputSamplerPoint, xy );
   pixel.a *= Opacity;

   return pixel;
}

float3x3 mYUV709n = { // Normalized
   0.2126, 0.7152, 0.0722,
   -0.1145721060573399, -0.3854278939426601, 0.5,
   0.5, -0.4541529083058166, -0.0458470916941834
};

// Colour Y Value Cr Value Cb Value
// White 235 (EB)    128 (80) 128 (80)
// Black 16 (10)     128 (80) 128 (80)
// Red 81 (51)       240 (F0) 90  (5A)
// Green 145 (91)    34 (22)  54  (36)
// Blue 41 (29)      110 (6E) 240 (F0)
// Yellow 210 (D2)   146 (92) 16  (10)
// Cyan 170 (AA)     16 (10)  166 (A6)
// Magenta 106 (6A)  222 (DE) 202 (CA)
//                   Y1             Cr                Y2                Cb
float4 White   = { 235 / 256.0,     0.5,              235.0 / 256.0,    0.5            };
float4 Black   = { 16  / 256.0,     0.5,              16 / 256.0,       0.5            };
float4 Red     = { 81  / 256.0,     240 / 256.0,      81 / 256.0,       90 / 256.0     };
float4 Green   = { 145 / 256.0,     34  / 256.0,      145 / 256.0,      54  / 256.0    };
float4 Blue    = { 41  / 256.0,     110 / 256.0,      41  / 256.0,      240 / 256.0    };
float4 Yellow  = { 210 / 256.0,     146 / 256.0,      210 / 256.0,      16  / 256.0    };
float4 Cyan    = { 170 / 256.0,     16  / 256.0,      170 / 256.0,      166 / 256.0    };
float4 Magenta = { 106 / 256.0,     222 / 256.0,      106 / 256.0,      202 / 256.0    };

float4 ColourBars_main( float2 xy : TEXCOORD1 ) : COLOR
{
   float4 ret;

   //int pix = (int) fmod( xy.x * _Width, _Width / 8 );
   int colour = (int) ( xy.x * 8 );

   if ( colour == 0 )
      ret = White;
   else if ( colour == 1 )
      ret = Black;
   else if ( colour == 2 )
      ret = Red;
   else if ( colour == 3 )
      ret = Green;
   else if ( colour == 4 )
      ret = Blue;
   else if ( colour == 5 )
      ret = Yellow;
   else if ( colour == 6 )
      ret = Cyan;
   else if ( colour == 7 )
      ret = Magenta;

   return ret;
}

// Note that :
// 1. The Y value ranges from 16 to 235 (220 levels), with 16 being black.
// 2. The Cr and Cb values range from 16 to 240 with 128 as the mid - point.
// 3. The above are for 8 - bit values. 10 - bit values can be acheived by multiplying the decimal values by 4.
// 4. The values in brackets are in hexadecimal.

float4 ARGBtoYUV_main( float2 xy : TEXCOORD1 ) : COLOR
{
   float4 in_rgba = tex2D( InputSamplerPoint, xy );

   float4 ret;
   
   int pix = (int) fmod( xy.x * _Width, _Width / 8 );

   if ( pix == 0 )
      ret = pix / 65536.0;
   else if ( pix == 1 )
      ret = pix / 65536.0;
   else if ( pix == 2 )
      ret = pix / 65536.0;
   else if ( pix == 3 )
      ret = pix / 65536.0;
   else if ( pix == 4 )
      ret = pix / 65536.0;
   else if ( pix == 5 )
      ret = pix / 65536.0;
   else if ( pix == 6 )
      ret = pix / 65536.0;
   else if ( pix == 7 )
      ret = pix / 65536.0;

   return ret;

   float4 ret2;
   ret.bag = ret.bag;
   ret.r = ret.b;
   return ret;

   ret = float4
   (
   0,
   0.25,
   0.5,
   0.75
   );
   
   //ret = float4
   //(
   //   in_rgba.r * mYUV709n._m00 + in_rgba.g * mYUV709n._m01 + in_rgba.b * mYUV709n._m02,
   //   in_rgba.r * mYUV709n._m10 + in_rgba.g * mYUV709n._m11 + in_rgba.b * mYUV709n._m12,
   //   in_rgba.r * mYUV709n._m20 + in_rgba.g * mYUV709n._m21 + in_rgba.b * mYUV709n._m22,
   //   in_rgba.a
   //) + float4(0, 0.5, 0.5, 0);

   // First pixel (provides Y, Cr and Cb samples at this position)
   ret.bag = float3
      (
      in_rgba.r * mYUV709n._m00 + in_rgba.g * mYUV709n._m01 + in_rgba.b * mYUV709n._m02,
      in_rgba.r * mYUV709n._m10 + in_rgba.g * mYUV709n._m11 + in_rgba.b * mYUV709n._m12,
      in_rgba.r * mYUV709n._m20 + in_rgba.g * mYUV709n._m21 + in_rgba.b * mYUV709n._m22
      ) + float3(0, 0.5, 0.5);


   // Second pixel (Provides Y-sample at this position only)
   ret.r = in_rgba.r * mYUV709n._m00 + in_rgba.g * mYUV709n._m01 + in_rgba.b * mYUV709n._m02;

   return ret;
}
//--------------------------------------------------------------//
// Techniques
//--------------------------------------------------------------//
technique Null                                 { pass P1 { PixelShader = compile PROFILE null_main(); } }
technique ReverseComponentOrder                { pass P1 { PixelShader = compile PROFILE revComp_main(); } }
technique FlipVertical                         { pass P1 { PixelShader = compile PROFILE flip_main(); } }
technique ReverseComponentOrderAndFlipVertical { pass P1 { PixelShader = compile PROFILE revAndFlip_main(); } }
technique UYVYtoYUYV                           { pass P1 { PixelShader = compile PROFILE UYVYtoYUYV_main(); } }
technique YVYUtoYUYV                           { pass P1 { PixelShader = compile PROFILE YVYUtoYUYV_main(); } }
technique YV12toYUYV                           { pass P1 { PixelShader = compile PROFILE YV12toYUYV_main(); } }
technique RotateLeft                           { pass P1 { PixelShader = compile PROFILE RotateLeft_main(); } }
technique Rotateright                          { pass P1 { PixelShader = compile PROFILE RotateRight_main(); } }
technique RGBtoARGB                            { pass P1 { PixelShader = compile PROFILE RGBtoARGB_main(); } }
technique RGB16toABGR                          { pass P1 { PixelShader = compile PROFILE RGB16toABGR_main(); } }
technique BGRtoARGB                            { pass P1 { PixelShader = compile PROFILE BGRtoARGB_main(); } }
technique RGBtoARGB_FlipVertical               { pass P1 { PixelShader = compile PROFILE RGBtoARGBFlip_main(); } }
technique RGBPlanartoARGB                      { pass P1 { PixelShader = compile PROFILE RGBPlanartoARGB_main(); } }
technique ChannelExpand                        { pass P1 { PixelShader = compile PROFILE ChannelExpand_main(); } }
technique Blend                                { pass P1 { PixelShader = compile PROFILE blend_main(); } }
technique V210toYUYV                           { pass P1 { PixelShader = compile PROFILE V210toYUYV_main(); } }
technique ARGBtoYUV                            { pass P1 { PixelShader = compile PROFILE ARGBtoYUV_main(); } }
technique ColourBars                           { pass P1 { PixelShader = compile PROFILE ColourBars_main(); } }
technique ARGBtoRGB                            { pass P1 { PixelShader = compile PROFILE ARGBtoRGB_main(); } }
technique ARGBtoBGR                            { pass P1 { PixelShader = compile PROFILE ARGBtoBGR_main(); } }
technique ARGBtoBGRA                           { pass P1 { PixelShader = compile PROFILE ARGBtoBGRA_main(); } }
technique BGRAtoARGB                           { pass P1 { PixelShader = compile PROFILE BGRAtoARGB_main(); } }
technique ABGRtoARGB                           { pass P1 { PixelShader = compile PROFILE ABGRtoARGB_main(); } }
