//--------------------------------------------------------------//
// Push.fx
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Push";
   string Category    = "Mixes";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
int SetTechnique
<
   string Description = "Type";
   string Enum = "Push Right,Push Down,Push Left,Push Up";
> = 0;

float Progress
<
   string Description = "Progress";
   float MinVal = 0.0;
   float MaxVal = 1.0;
   float KF0    = 0.0;
   float KF1    = 1.0;
> = 0.0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture fg;
texture bg;
sampler FgSampler = sampler_state { Texture = <fg>; };
sampler BgSampler = sampler_state { Texture = <bg>; };

float _bgNormWidth = 1.0;

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 Push1_main( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 ret;

   if ( xyNorm.x > Progress )
   {
      ret = tex2D( FgSampler, float2( xy1.x - Progress, xy1.y ) );
   }
   else
   {
      ret = tex2D( BgSampler, float2( _bgNormWidth - ( Progress - xy2.x ), xy2.y )  );
   }

   return ret;
}

//--------------------------------------------------------------//
float4 Push2_main( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 ret;

   if ( xyNorm.y > Progress )
   {
      ret = tex2D( FgSampler, float2( xy1.x, xy1.y - Progress ) );
   }
   else
   {
      ret = tex2D( BgSampler, float2( xy2.x, 1.0 - ( Progress - xy2.y ) )  );
   }

   return ret;
}

//--------------------------------------------------------------//
float4 Push3_main( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 ret;

   // Same as push1 except going the other way, so :
   float progress = 1.0 - Progress;

   if ( xyNorm.x > progress )
   {
      ret = tex2D( BgSampler, float2( xy1.x - progress, xy1.y ) );
   }
   else
   {
      ret = tex2D( FgSampler, float2( _bgNormWidth - ( progress - xy2.x ), xy2.y )  );
   }

   return ret;
}

//--------------------------------------------------------------//
float4 Push4_main( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 ret;

   // Same as push2 except going the other way, so :
   float progress = 1.0 - Progress;

   if ( xyNorm.y > progress )
   {
      ret = tex2D( BgSampler, float2( xy1.x, xy1.y - progress ) );
   }
   else
   {
      ret = tex2D( FgSampler, float2( xy2.x, 1.0 - ( progress - xy2.y ) )  );
   }

   return ret;
}

//--------------------------------------------------------------//
// Techniques
//--------------------------------------------------------------//
technique Push1 { pass SinglePass { PixelShader = compile PROFILE Push1_main(); } }
technique Push2 { pass SinglePass { PixelShader = compile PROFILE Push2_main(); } }
technique Push3 { pass SinglePass { PixelShader = compile PROFILE Push3_main(); } }
technique Push4 { pass SinglePass { PixelShader = compile PROFILE Push4_main(); } }
