//--------------------------------------------------------------//
// Resize
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "Internal";
> = 0;

//--------------------------------------------------------------//
// Inputs/Samplers
//--------------------------------------------------------------//
texture Input;

sampler LinearSampler = sampler_state
{
   Texture = <Input>;
   MipFilter = LINEAR;
   MinFilter = LINEAR;
   MagFilter = LINEAR;
};

// Frame height, in pixels
float gOutputHeight = 1.0;

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 upperRasterFieldDouble_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   // Pre-shader bits
   float sourceHeight         = gOutputHeight / 2.0;
   float sourceLineHeight     = 1.0 / sourceHeight;
   float sourceHalfLineHeight = sourceLineHeight / 2.0;

   // Calc line index
   float destLineIdx = floor( ( xy1.y * gOutputHeight ) + 0.5 );

   return tex2D( LinearSampler, float2( xy1.x, ( sourceLineHeight * ( destLineIdx / 2.0 ) ) + sourceHalfLineHeight  ) );
}
//--------------------------------------------------------------//
float4 lowerRasterFieldDouble_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   // Pre-shader bits
   float sourceHeight         = gOutputHeight / 2.0;
   float sourceLineHeight     = 1.0 / sourceHeight;
   float sourceHalfLineHeight = sourceLineHeight / 2.0;

   // Calc line index
   float destLineIdx = floor( ( xy1.y * gOutputHeight ) + 0.5 );

   return tex2D( LinearSampler, float2( xy1.x, ( sourceLineHeight * ( ( destLineIdx - 1.0 ) / 2.0 ) ) + sourceHalfLineHeight  ) );
}
//--------------------------------------------------------------//
const float gImageShiftFactor = 0.0;

float4 upperRasterReduce_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float lineHeight = 1.0 / gOutputHeight;

   return tex2D( LinearSampler, float2( xy1.x, xy1.y + ( gImageShiftFactor * lineHeight ) ) );
}

//--------------------------------------------------------------//
float4 lowerRasterReduce_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float lineHeight = 1.0 / gOutputHeight;

   return tex2D( LinearSampler, float2( xy1.x, xy1.y - ( gImageShiftFactor * lineHeight ) ) );
}

//--------------------------------------------------------------//
// Techniques
//--------------------------------------------------------------//
technique UpperRasterFieldDouble { pass p1 { PixelShader = compile PROFILE upperRasterFieldDouble_main(); } }
technique LowerRasterFieldDouble { pass p1 { PixelShader = compile PROFILE lowerRasterFieldDouble_main(); } }
technique UpperRasterReduce      { pass p1 { PixelShader = compile PROFILE upperRasterReduce_main(); } }
technique LowerRasterReduce      { pass p1 { PixelShader = compile PROFILE lowerRasterReduce_main(); } }
