//--------------------------------------------------------------//
// squeeze.fx
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Squeeze";
   string Category    = "Mixes";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
int SetTechnique
<
   string Description = "Type";
   string Enum = "Squeeze Right,Squeeze Down,Squeeze Left,Squeeze Up";
> = 0;

float Progress
<
   string Description = "Progress";
   float MinVal = 0.0;
   float MaxVal = 1.0;
   float KF0    = 0.0;
   float KF1    = 1.0;
> = 0.0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture fg;
texture bg;
sampler FgSampler = sampler_state { Texture = <fg>; };
sampler BgSampler = sampler_state { Texture = <bg>; };

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 Squeeze1_main( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 ret;

   if ( xyNorm.x > Progress )
   {
      ret = tex2D( FgSampler, float2( ( xy1.x - Progress ) / ( 1.0 - Progress ), xy1.y )  );
   }
   else
   {
      ret = tex2D( BgSampler, float2( xy2.x / Progress, xy2.y )  );
   }

   return ret;
}

//--------------------------------------------------------------//
float4 Squeeze2_main( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 ret;

   if ( xyNorm.y > Progress )
   {
      ret = tex2D( FgSampler, float2( xy1.x, ( xy1.y - Progress ) / ( 1.0 - Progress ) ) );
   }
   else
   {
      ret = tex2D( BgSampler, float2( xy2.x, xy2.y / Progress ) );
   }

   return ret;
}

//--------------------------------------------------------------//
float4 Squeeze3_main( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 ret;

   if ( xyNorm.x > ( 1.0 - Progress ) )
   {
      ret = tex2D( BgSampler, float2( ( xy2.x - ( 1.0 - Progress ) ) / Progress, xy2.y )  );
   }
   else
   {
      ret = tex2D( FgSampler, float2( xy1.x / ( 1.0 - Progress ), xy1.y ) );
   }

   return ret;
}

//--------------------------------------------------------------//
float4 Squeeze4_main( float2 xyNorm : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float4 ret;

   if ( xyNorm.y > ( 1.0 - Progress ) )
   {
      ret = tex2D( BgSampler, float2( xy2.x ,( xy2.y - ( 1.0 - Progress ) ) / Progress ) );
   }
   else
   {
      ret = tex2D( FgSampler, float2( xy1.x, xy1.y / ( 1.0 - Progress ) ) );
   }

   return ret;
}

//--------------------------------------------------------------//
// Techniques
//--------------------------------------------------------------//
technique Squeeze1 { pass SinglePass { PixelShader = compile PROFILE Squeeze1_main(); } }
technique Squeeze2 { pass SinglePass { PixelShader = compile PROFILE Squeeze2_main(); } }
technique Squeeze3 { pass SinglePass { PixelShader = compile PROFILE Squeeze3_main(); } }
technique Squeeze4 { pass SinglePass { PixelShader = compile PROFILE Squeeze4_main(); } }
