//--------------------------------------------------------------//
// TexWipe.fx
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Texture Wipe";
   string Category    = "Mixes";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//

float XRepeat
<
   string Description = "Across";
   string Group = "Repeats";
   float MinVal = 0.5;
   float MaxVal = 50.00;
> = 1.0;

float YRepeat
<
   string Description = "Down";
   string Group = "Repeats";
   float MinVal = 0.5;
   float MaxVal = 50.00;
> = 1.0;

float BorderWidth
<
   string Description = "Size";
   string Group = "Border";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.0;

float4 BorderColour1
<
   string Description = "Colour one";
   string Group       = "Border";
> = float4( 1.0, 1.0, 0.0, 1.0 );

float4 BorderColour2
<
   string Description = "Colour two";
   string Group       = "Border";
> = float4( 1.0, 0.0, 0.0, 1.0 );

float Softness
<
   string Description = "Softness";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.2;

float Progress
<
   string Description = "Progress";
   float MinVal = 0.0;
   float MaxVal = 1.0;
   float KF0    = 0.0;
   float KF1    = 1.0;
> = 0.0;

bool Invert
<
   string Description = "Reverse";
> = false ;

float XPos
<
   string Description = "Horizontal Position";
   string Group       = "Position";
   float MinVal = -0.5;
   float MaxVal = 1.5;
> = 0.5;

float YPos
<
   string Description = "Vertical Position";
   string Group       = "Position";
   float MinVal = -0.5;
   float MaxVal = 1.5;
> = 0.5;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture fg;
texture bg;
texture _tex1;
texture _tex2;
texture tex2 : RenderColorTarget;

//--------------------------------------------------------------//
// Samplers
//--------------------------------------------------------------//
sampler FgSampler  = sampler_state { Texture = <fg>; };
sampler BgSampler  = sampler_state { Texture = <bg>; };
sampler TexSamplerLinear = sampler_state
{
   Texture = <_tex1>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;
   ADDRESSU  = ClampToEdge;
   ADDRESSV  = ClampToEdge;
};
sampler TexSamplerPoint = sampler_state
{
   Texture = <_tex2>;
   MINFILTER = POINT;
   MIPFILTER = POINT;
   MAGFILTER = POINT;
   ADDRESSU  = ClampToEdge;
   ADDRESSV  = ClampToEdge;
};

sampler Tex2Sampler = sampler_state
{
   Texture = <tex2>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;
   ADDRESSU  = ClampToEdge;
   ADDRESSV  = ClampToEdge;
};
/*
// The following variables are filled in by the host, but can be calculated from the public
// params above - its done outside of the shader code to cut down on shader instructions
float _TotalEdgeSize;
float _HalfSoftness;
float _EdgeSoftStart;
float _BorderStart;
float _BorderEnd;
float _EdgeSoftEnd;
*/

float _OutputWidth  = 1.0;
float _OutputHeight = 1.0;
float _OutputAspectRatio = 1.0;

// Private constants
const float  _PI         = 3.1415926535;
const float  _TwoPI      = 2.0 * 3.1415926535;
const float  _HalfPI     = 3.1415926535 / 2.0;
const float  _QuarterPI  = 3.1415926535 / 4.0;
const float  _PIAndAHalf = 1.5 * 3.1415926535;
const float2 _Centre     = float2( 0.5, 0.5 );
const float _255         = 255.0;
const float _sqrt2       = sqrt( 2.0 );
const float _EllipseSquish       = 0.6;
const float _Tan60               = 1.7320; //tan( 2 * 3.14159 / 6 );
const float _Tan30               = 0.5774; //tan( 2 * 3.14159 / 12 );
const float _Cos30               = 0.8660; //cos( 2 * 3.14159 / 12 );
const float _SixtyDegreeGradient = 0.5774; //1.0 / tan( 2 * 3.14159 / 6 );

//---------------------------------------------------------------------------------------//
// doWipe
//
// Main wipe rendering code (shared between all three wipe variants -
// matrix, bitmap & smpte)
//---------------------------------------------------------------------------------------//
float4 doWipe( float2 xy1, float2 xy2, float luma )
{
   float totalEdgeSize = Softness + BorderWidth;
   float halfSoftness  = ( Softness / 2.0 );
   float edgeSoftStart = ( Progress * ( 1.0 + totalEdgeSize ) ) - totalEdgeSize;
   float borderStart   = edgeSoftStart + halfSoftness;
   float borderEnd     = borderStart + BorderWidth;
   float edgeSoftEnd   = borderEnd + halfSoftness;

   float4 ret;
   float4 fgPix = tex2D( FgSampler, xy1 );
   float4 bgPix = tex2D( BgSampler, xy2 );

   luma = lerp( luma, 1.0 - luma, Invert );

   if ( luma > edgeSoftEnd )
   {
      ret = fgPix;
   }
   else if ( luma < edgeSoftStart )
   {
      ret = bgPix;
   }
   else
   {
      float lumaMinusEdgeStart =  luma - edgeSoftStart;

      if ( BorderWidth == 0 )
      {
         ret = lerp( bgPix, fgPix, lumaMinusEdgeStart / Softness );
      }
      else
      {
         float4 borderColour = lerp( BorderColour1, BorderColour2, lumaMinusEdgeStart / totalEdgeSize );

         if ( luma < borderStart )
            ret = lerp( bgPix, borderColour, lumaMinusEdgeStart / halfSoftness );
         else if ( luma > borderEnd )
            ret = lerp( fgPix, borderColour, ( edgeSoftEnd - luma ) / halfSoftness );
         else
            ret = borderColour;
      }
   }

   return ret;
}
//---------------------------------------------------------------------------------------//
// linearSampledWipe_main()
//
// Used for arbitrary bitmap wipes
//---------------------------------------------------------------------------------------//
float4 linearSampledWipe_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   // Calc texture address (based on x/y repeats)
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float4 texPix = tex2D( TexSamplerLinear, xy0 );
   return doWipe( xy1, xy2, ( texPix.r + texPix.g + texPix.b ) / 3.0 );
}
//---------------------------------------------------------------------------------------//
// pointSampledWipe_main()
//
// Used for matrix wipes
//---------------------------------------------------------------------------------------//
float4 pointSampledWipe_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   // Calc texture address (based on x/y repeats)
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float4 texPix = tex2D( TexSamplerPoint, xy0 );
   return doWipe( xy1, xy2, ( texPix.r + texPix.g + texPix.b ) / 3.0 );
}

//---------------------------------------------------------------------------------------//
// Pixel shaders for SMPTE wipes
//---------------------------------------------------------------------------------------//
float4 Bar_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, xy0.x );
}
//--------------------------------------------------------------//
float4 Bar_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, xy0.y );
}
//--------------------------------------------------------------//
float4 Box_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, max( xy0.x, xy0.y ) );
}
//--------------------------------------------------------------//
float4 Box_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, max( 1.0 - xy0.x, xy0.y ) );
}
//--------------------------------------------------------------//
float4 Box_3_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, max( 1.0 - xy0.x, 1.0 - xy0.y ) );
}
//--------------------------------------------------------------//
float4 Box_4_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, max( xy0.x, 1.0 - xy0.y ) );
}
//--------------------------------------------------------------//
float4 Box_5_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, max( abs( xy0.x - float2(XPos, YPos).x ) * 2.0 , xy0.y ) );
}
//--------------------------------------------------------------//
float4 Box_6_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, max( 1.0 - xy0.x, abs( xy0.y - float2(XPos, YPos).y ) * 2.0 ) );
}
//--------------------------------------------------------------//
float4 Box_7_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, max( abs( xy0.x - float2(XPos, YPos).x ) * 2.0, 1.0 - xy0.y ) );
}
//--------------------------------------------------------------//
float4 Box_8_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, max( abs( xy0.y - float2(XPos, YPos).y ) * 2.0, xy0.x ) );
}
//--------------------------------------------------------------//
float4 Four_Box_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 = abs( float2(XPos, YPos) - xy0 ) * 2.0;
   return doWipe( xy1, xy2, 1.0 - min( xy0.x, xy0.y ) );
}
//--------------------------------------------------------------------------
float4 Four_Box_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, max( abs( frac( xy0.x * 2.0 ) - float2(XPos, YPos).x ) * 2.0,
                         abs( frac( xy0.y * 2.0 ) - float2(XPos, YPos).y ) * 2.0 ) );
}
//--------------------------------------------------------------------------
float4 Barn_Door_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, abs( xy0.x - float2(XPos, YPos).x ) * 2.0 );
}
//--------------------------------------------------------------------------
float4 Barn_Door_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, abs( xy0.y - float2(XPos, YPos).y ) * 2.0 );
}
//--------------------------------------------------------------------------
float4 Barn_Door_3_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float c = xy0.x + xy0.y;

   if ( c >= 1.0 )
      c = 1.0 - frac( c );

   return doWipe( xy1, xy2, c );
}
//--------------------------------------------------------------------------
float4 Barn_Door_4_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float c = ( 1.0 - xy0.x ) + xy0.y;

   if ( c >= 1.0 )
      c = 1.0 - frac( c );

   return doWipe( xy1, xy2, c );
}
//--------------------------------------------------------------------------
float4 Diagonal_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, ( xy0.x + xy0.y ) / 2.0 );
}
//--------------------------------------------------------------------------
float4 Diagonal_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, ( ( 1.0 - xy0.x ) + xy0.y ) / 2.0 );
}
//--------------------------------------------------------------------------
float4 Bow_Tie_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float c;

   if ( xy0.x < float2(XPos, YPos).x && xy0.y < float2(XPos, YPos).y )
      c = ( float2(XPos, YPos).x - xy0.x ) + xy0.y;

   else if ( xy0.x >= float2(XPos, YPos).x && xy0.y < float2(XPos, YPos).y )
      c = ( 1.0 - ( 1.0 - ( xy0.x - float2(XPos, YPos).x ) ) ) + xy0.y;

   else if ( xy0.x < float2(XPos, YPos).x && xy0.y >= float2(XPos, YPos).y )
      c = ( float2(XPos, YPos).x - xy0.x ) + ( 1.0 - xy0.y );
   else
      c = ( 1.0 - ( 1.0 - ( xy0.x - float2(XPos, YPos).x ) ) ) + ( 1.0 - xy0.y );

   return doWipe( xy1, xy2, c );
}
//--------------------------------------------------------------------------
float4 Vee_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float c;

   if ( xy0.x < float2(XPos, YPos).x )
      c = ( ( ( float2(XPos, YPos).x - xy0.x ) * 2.0 ) + xy0.y ) / 2.0;

   else
      c = ( ( ( xy0.x - float2(XPos, YPos).x ) * 2.0 ) + xy0.y ) / 2.0;

   return doWipe( xy1, xy2, c );
}
//--------------------------------------------------------------------------
float4 Vee_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0.x = 1.0 - xy0.x;

   float c;

   if ( xy0.y < float2(XPos, YPos).y )
      c = ( ( ( float2(XPos, YPos).y - xy0.y ) * 2.0 ) + xy0.x ) / 2.0;

   else
      c = ( ( ( xy0.y - float2(XPos, YPos).y ) * 2.0 ) + xy0.x ) / 2.0;

   return doWipe( xy1, xy2, c );
}
//--------------------------------------------------------------------------
float4 Vee_3_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0.y = 1.0 - xy0.y;

   float c;

   if ( xy0.x < float2(XPos, YPos).x )
      c = ( ( ( float2(XPos, YPos).x - xy0.x ) * 2.0 ) + xy0.y ) / 2.0;

   else
      c = ( ( ( xy0.x - float2(XPos, YPos).x ) * 2.0 ) + xy0.y ) / 2.0;

   return doWipe( xy1, xy2, c );
}
//--------------------------------------------------------------------------
float4 Vee_4_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float c;

   if ( xy0.y < float2(XPos, YPos).y )
      c = ( ( ( float2(XPos, YPos).y - xy0.y ) * 2.0 ) + xy0.x ) / 2.0;

   else
      c = ( ( ( xy0.y - float2(XPos, YPos).y ) * 2.0 ) + xy0.x ) / 2.0;

   return doWipe( xy1, xy2, c );
}
//--------------------------------------------------------------------------
float4 Barn_Vee_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0.y = 1.0 - xy0.y;

   float c;

   if ( xy0.x < float2(XPos, YPos).x )
   {
      c = ( 2.0 * xy0.x ) + xy0.y;

      if ( c > 1.0 )
         c = 1.0 - frac( c );
   }
   else
   {
      c = ( ( 1.0 - xy0.x ) * 2.0 ) + xy0.y;

      if ( c > 1.0 )
         c = 1.0 - frac( c );
   }

   return doWipe( xy1, xy2, c );
}
//--------------------------------------------------------------------------
float4 Barn_Vee_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float c;

   if ( xy0.y < float2(XPos, YPos).y )
   {
      c = ( 2.0 * xy0.y ) + xy0.x;

      if ( c > 1.0 )
         c = 1.0 - frac( c );
   }
   else
   {
      c = ( ( 1.0 - xy0.y ) * 2.0 ) + xy0.x;

      if ( c > 1.0 )
         c = 1.0 - frac( c );
   }

   return doWipe( xy1, xy2, c );
}
//--------------------------------------------------------------------------
float4 Barn_Vee_3_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float c;

   if ( xy0.x < float2(XPos, YPos).x )
   {
      c = ( 2.0 * xy0.x ) + xy0.y;

      if ( c > 1.0 )
         c = 1.0 - frac( c );
   }
   else
   {
      c = ( ( 1.0 - xy0.x ) * 2.0 ) + xy0.y;

      if ( c > 1.0 )
         c = 1.0 - frac( c );
   }

   return doWipe( xy1, xy2, c );
}
//--------------------------------------------------------------------------
float4 Barn_Vee_4_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0.x = 1.0 - xy0.x;

   float c;

   if ( xy0.y < float2(XPos, YPos).y )
   {
      c = ( 2.0 * xy0.y ) + xy0.x;

      if ( c > 1.0 )
         c = 1.0 - frac( c );
   }
   else
   {
      c = ( ( 1.0 - xy0.y ) * 2.0 ) + xy0.x;

      if ( c > 1.0 )
         c = 1.0 - frac( c );
   }

   return doWipe( xy1, xy2, c );
}
//--------------------------------------------------------------------------
float4 Clock_6_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 -= float2(XPos, YPos);

   float angleRads = atan( abs( xy0.y / xy0.x ) );

   if ( xy0.x >= 0 )
   {
      if ( xy0.y < 0 )
         angleRads = _PIAndAHalf - angleRads;
      else
         angleRads += _PIAndAHalf;
   }
   else
   {
      if ( xy0.y < 0 )
         angleRads += _HalfPI;
      else
         angleRads = ( _HalfPI - angleRads );
   }

   return doWipe( xy1, xy2, angleRads / _TwoPI );
}
//--------------------------------------------------------------------------
float4 Clock_12_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 -= float2(XPos, YPos);

   float angleRads = atan( abs( xy0.y / xy0.x ) );

   if ( xy0.x >= 0 )
   {
      if ( xy0.y < 0 )
         angleRads = ( _HalfPI - angleRads );
      else
         angleRads += _HalfPI;
   }
   else
   {
      if ( xy0.y < 0 )
         angleRads += _PIAndAHalf;
      else
         angleRads = _PIAndAHalf - angleRads;
   }

   return doWipe( xy1, xy2, angleRads / _TwoPI );
}
//--------------------------------------------------------------------------
float4 Clock_3_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 -= float2(XPos, YPos);

   float angleRads = atan( abs( xy0.y / xy0.x ) );

   if ( xy0.x >= 0 )
   {
      if ( xy0.y < 0 )
         angleRads = _TwoPI - angleRads;
   }
   else
   {
      if ( xy0.y >= 0 )
         angleRads = _PI - angleRads;
      else
         angleRads += _PI;
   }

   return doWipe( xy1, xy2, angleRads / _TwoPI );
}
//--------------------------------------------------------------------------
float4 Clock_9_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 -= float2(XPos, YPos);

   float angleRads = atan( abs( xy0.y / xy0.x ) );

   if ( xy0.x >= 0 )
   {
      if ( xy0.y >= 0 )
         angleRads += _PI;
      else
         angleRads = _PI - angleRads;
   }
   else
   {
      if ( xy0.y >= 0 )
         angleRads = _TwoPI - angleRads;
   }

   return doWipe( xy1, xy2, angleRads / _TwoPI );
}
//--------------------------------------------------------------------------
float4 PinWheel_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 -= float2(XPos, YPos);

   float angleRads = atan( abs( xy0.y / xy0.x ) );

   if ( ( xy0.y < 0.0 && xy0.x < 0.0 ) || ( xy0.x >= 0.0 && xy0.y >= 0.0 ) )
   {
      angleRads += _HalfPI;
   }
   else
   {
      angleRads = _HalfPI - angleRads;
   }

   return doWipe( xy1, xy2, angleRads / _PI );
}
//--------------------------------------------------------------------------
float4 PinWheel_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 -= float2(XPos, YPos);

   float angleRads = atan( abs( xy0.x / xy0.y ) );

   if ( ( xy0.y >= 0.0 && xy0.x < 0.0 ) || ( xy0.x >= 0.0 && xy0.y < 0.0 ) )
   {
      angleRads += _HalfPI;
   }
   else
   {
      angleRads = _HalfPI - angleRads;
   }

   return doWipe( xy1, xy2, angleRads / _PI );
}
//--------------------------------------------------------------------------
float4 PinWheel_3_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 -= float2(XPos, YPos);

   float angleRads = atan( abs( xy0.y / xy0.x ) );

   if ( ( xy0.y < 0.0 && xy0.x >= 0.0 ) || ( xy0.x < 0.0 && xy0.y >= 0.0 ) )
   {
      angleRads = _HalfPI - angleRads;
   }

   return doWipe( xy1, xy2, angleRads / _HalfPI );
}
//--------------------------------------------------------------------------
float4 Single_Sweep_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float angleRads = abs( atan( xy0.y / ( xy0.x - float2(XPos, YPos).x ) ) );

   if ( xy0.x < float2(XPos, YPos).x )
      angleRads = _PI - angleRads;

   return doWipe( xy1, xy2, angleRads / _PI );
}
//--------------------------------------------------------------------------
float4 Single_Sweep_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 = 1.0 - xy0;

   float angleRads = _HalfPI - abs( atan( ( xy0.y - float2(XPos, YPos).y ) / xy0.x ) );

   if ( xy0.y >= float2(XPos, YPos).y )
      angleRads = _PI - angleRads;

   return doWipe( xy1, xy2, angleRads / _PI );
}
//--------------------------------------------------------------------------
float4 Single_Sweep_3_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 = 1.0 - xy0;

   float angleRads = abs( atan(  xy0.y / ( xy0.x - float2(XPos, YPos).x ) ) );

   if ( xy0.x < float2(XPos, YPos).x )
      angleRads = _PI - angleRads;

   return doWipe( xy1, xy2, angleRads / _PI );
}
//--------------------------------------------------------------------------
float4 Single_Sweep_4_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float angleRads = _HalfPI - abs( atan( ( xy0.y - float2(XPos, YPos).y ) / xy0.x ) );

   if ( xy0.y >= float2(XPos, YPos).y )
      angleRads = _PI - angleRads;

   return doWipe( xy1, xy2, angleRads / _PI );
}
//--------------------------------------------------------------------------
float4 Single_Sweep_5_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, abs( atan( xy0.x / xy0.y ) ) / _HalfPI );
}
//--------------------------------------------------------------------------
float4 Single_Sweep_6_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, abs( atan( ( 1.0 - xy0.y ) / xy0.x ) ) / _HalfPI );
}
//--------------------------------------------------------------------------
float4 Single_Sweep_7_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, abs( atan( ( 1.0 - xy0.y ) / ( 1.0 - xy0.x ) ) ) / _HalfPI );
}
//--------------------------------------------------------------------------
float4 Single_Sweep_8_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, abs( atan( xy0.y / ( 1.0 - xy0.x ) ) ) / _HalfPI );
}
//--------------------------------------------------------------------------
float4 Double_Sweep_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float xDistFromCentre = xy0.x - float2(XPos, YPos).x;
   float yDistFromCentre = xy0.x >= float2(XPos, YPos).x ? xy0.y : 1.0 - xy0.y;

   return doWipe( xy1, xy2, abs( atan( yDistFromCentre / xDistFromCentre ) ) / _HalfPI );
}
//--------------------------------------------------------------------------
float4 Double_Sweep_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0.y = 1.0 - xy0.y;

   float xDistFromCentre = ( xy0.y >= float2(XPos, YPos).y ) ? xy0.x : 1.0 - xy0.x;
   float yDistFromCentre = xy0.y - float2(XPos, YPos).y;

   return doWipe( xy1, xy2, 1.0 - ( abs( atan( yDistFromCentre / xDistFromCentre ) ) / _HalfPI ) );
}
//--------------------------------------------------------------------------
float4 Double_Sweep_3_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0.x = 1.0 - xy0.x;

   float xDistFromCentre = xy0.x - float2(XPos, YPos).x;
   float yDistFromCentre = xy0.y < float2(XPos, YPos).y ? xy0.y : 1.0 - xy0.y;

   float angleRads = abs( atan( yDistFromCentre / xDistFromCentre ) );

   if ( xy0.x >= float2(XPos, YPos).x )
   {
      angleRads = _PI - angleRads;
   }

   return doWipe( xy1, xy2, angleRads / _PI );
}
//--------------------------------------------------------------------------
float4 Double_Sweep_4_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0.y = 1.0 - xy0.y;

   float yDistFromCentre = xy0.y - float2(XPos, YPos).y;
   float xDistFromCentre = xy0.x < float2(XPos, YPos).x ? xy0.x : 1.0 - xy0.x;

   float angleRads = atan( abs( yDistFromCentre / xDistFromCentre ) );

   if ( xy0.y >= float2(XPos, YPos).y )
   {
      angleRads = _HalfPI - angleRads;
   }
   else
   {
      angleRads += _HalfPI;
   }

   return doWipe( xy1, xy2, angleRads / _PI );
}
//--------------------------------------------------------------------------
float4 Double_Sweep_5_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float f;

   if ( ( ( 1.0 - xy0.x ) + xy0.y ) > 1.0 )
      f = ( 1.0 - xy0.y ) / ( 1.0 - xy0.x );
   else
      f =  xy0.y / xy0.x;

   return doWipe( xy1, xy2, atan( abs( f ) ) / _QuarterPI );

}
//--------------------------------------------------------------------------
float4 Double_Sweep_6_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float f;

   if ( ( xy0.x + xy0.y ) > 1.0 )
      f = xy0.y / ( 1.0 - xy0.x );
   else
      f = ( 1.0 - xy0.y ) / xy0.x;

   return doWipe( xy1, xy2, ( _HalfPI - atan( abs( f ) ) ) / _QuarterPI );
}
//--------------------------------------------------------------------------
float4 Saloon_Door_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float xDistFromCentre = ( xy0.x < float2(XPos, YPos).x ? xy0.x : 1.0 - xy0.x ) * 2;

   return doWipe( xy1, xy2, abs( atan( xy0.y / xDistFromCentre ) ) / _HalfPI );
}
//--------------------------------------------------------------------------
float4 Saloon_Door_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float yDistFromCentre = ( xy0.y >= float2(XPos, YPos).y ? 1.0 - xy0.y : xy0.y ) * 2;
   float angleRads = abs( atan( yDistFromCentre / xy0.x ) );

   return doWipe( xy1, xy2, 1.0 - ( angleRads / _HalfPI ) );
}
//--------------------------------------------------------------------------
float4 Saloon_Door_3_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float xDistFromCentre = ( xy0.x < float2(XPos, YPos).x ? xy0.x : 1.0 - xy0.x ) * 2;
   float angleRads = abs( atan( ( 1.0 - xy0.y ) / xDistFromCentre ) );

   return doWipe( xy1, xy2, angleRads / _HalfPI );
}
//--------------------------------------------------------------------------
float4 Saloon_Door_4_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float yDistFromCentre = ( xy0.y >= float2(XPos, YPos).y ? 1.0 - xy0.y : xy0.y ) * 2;
   float angleRads = abs( atan( yDistFromCentre / ( 1.0 - xy0.x ) ) );

   return doWipe( xy1, xy2, 1.0 - ( angleRads / _HalfPI ) );
}
//--------------------------------------------------------------------------
float4 Windshield_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float xx = ( float2(XPos, YPos).x - xy0.x );
   float yy = ( xy0.y < float2(XPos, YPos).y ? ( xy0.y - 0.25 ) : ( ( 1.0 - xy0.y ) - 0.25 ) );

   float xDistFromCentre = abs( xx );
   float yDistFromCentre = abs( yy );

   float angleRads = atan( yDistFromCentre / xDistFromCentre );

   if ( xx >= 0 )
   {
      if ( yy < 0 )
         angleRads += _HalfPI;
      else
         angleRads = ( _HalfPI - angleRads );
   }
   else if ( xx < 0 )
   {
      if ( yy < 0 )
         angleRads = ( _HalfPI - angleRads ) + _PI;
      else
         angleRads += _PI + _HalfPI;
   }

   return doWipe( xy1, xy2, 1.0 - ( angleRads / _TwoPI ) );
}
//--------------------------------------------------------------------------
float4 Windshield_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float xx = ( xy0.x < float2(XPos, YPos).x ) ? ( xy0.x - 0.25 ) : ( ( 1.0 - xy0.x ) - 0.25 );
   float yy = xy0.y - float2(XPos, YPos).y;

   float angleRads = atan( abs( yy / xx ) );

   if ( xx >= 0 )
   {
      if ( yy >= 0 )
         angleRads = _TwoPI - angleRads;
   }
   else
   {
      if ( yy < 0 )
         angleRads = ( _HalfPI - angleRads ) + _HalfPI;
      else
         angleRads += _PI;
   }

   return doWipe( xy1, xy2, angleRads / _TwoPI );
}
//--------------------------------------------------------------------------
float4 Windshield_3_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float xDistFromCentre = ( xy0.x - float2(XPos, YPos).x );
   float yDistFromCentre = xy0.y < float2(XPos, YPos).y ? ( xy0.y - 0.25 ) : ( ( 1.0 - xy0.y ) - 0.25 );

   float angleRads = atan( abs( yDistFromCentre / xDistFromCentre ) );

   if ( xy0.y < 0.25 || ( xy0.y >= float2(XPos, YPos).y && xy0.y < 0.75 ) )
   {
      angleRads = _HalfPI - angleRads;
   }
   else
   {
      angleRads += _HalfPI;
   }

   float c = angleRads / _PI;

   if ( xy0.y >= float2(XPos, YPos).y )
      c = 1.0 - c;

   return doWipe( xy1, xy2, c );
}
//--------------------------------------------------------------------------
float4 Windshield_4_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   float xDistFromCentre = xy0.x < float2(XPos, YPos).x ? ( xy0.x - 0.25 ) : ( ( 1.0 - xy0.x ) - 0.25 );
   float yDistFromCentre = ( xy0.y - float2(XPos, YPos).y );

   float angleRads = atan( abs( xDistFromCentre / yDistFromCentre ) );

   if ( ( xy0.x >= 0.25 && xy0.x < float2(XPos, YPos).x ) || xy0.x >= 0.75 )
   {
      angleRads = _HalfPI - angleRads;
   }
   else
   {
      angleRads += _HalfPI;
   }

   float c = angleRads / _PI;

   if ( xy0.x < float2(XPos, YPos).x )
      c = 1.0 - c;

   return doWipe( xy1, xy2, c );
}
//--------------------------------------------------------------------------
float4 Double_Fan_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 -= float2(XPos, YPos);

   return doWipe( xy1, xy2, abs( atan( xy0.x / xy0.y ) ) / _HalfPI );
}
//--------------------------------------------------------------------------
float4 Fan_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 -= float2(XPos, YPos);

   float angleRads = abs( atan( xy0.y / xy0.x ) );

   if ( xy0.y < 0.0 )
   {
      angleRads = _HalfPI - angleRads;
   }
   else
   {
      angleRads += _HalfPI;
   }

   return doWipe( xy1, xy2, angleRads / _PI );
}
//--------------------------------------------------------------------------
float4 Fan_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 -= float2(XPos, YPos);

   float angleRads = abs( atan( xy0.x / xy0.y ) );

   if ( xy0.x >= 0.0 )
   {
      angleRads = _HalfPI - angleRads;
   }
   else
   {
      angleRads += _HalfPI;
   }

   return doWipe( xy1, xy2,  angleRads / _PI );
}
//--------------------------------------------------------------------------
float4 Fan_3_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, abs( atan( ( xy0.x - float2(XPos, YPos).x ) / xy0.y ) ) / _HalfPI );
}
//--------------------------------------------------------------------------
float4 Fan_4_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2,  1.0 - ( abs( atan( xy0.x / ( xy0.y - float2(XPos, YPos).y ) ) ) / _HalfPI ) );
}
//--------------------------------------------------------------------------
float4 Fan_5_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, abs( atan( ( xy0.x - float2(XPos, YPos).x ) / ( 1.0 - xy0.y ) ) ) / _HalfPI );
}
//--------------------------------------------------------------------------
float4 Fan_6_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   return doWipe( xy1, xy2, 1.0 - ( abs( atan( ( 1.0 - xy0.x ) / ( xy0.y - float2(XPos, YPos).y ) ) ) / _HalfPI ) );
}
//--------------------------------------------------------------------------
float4 Square_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 -= float2(XPos, YPos);
   xy0.y /=_OutputAspectRatio; // Effectively makes it square pixels
   xy0 = abs( xy0 ) * 2.0;

   return doWipe( xy1, xy2, max( xy0.x, xy0.y ) );
}
//--------------------------------------------------------------------------
float4 Diamond_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float maxVal = float2(XPos, YPos).x + ( float2(XPos, YPos).y / _OutputAspectRatio );

   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );
   xy0 -= float2(XPos, YPos);
   xy0.y /= _OutputAspectRatio; // Effectively makes it square pixels

   xy0 = abs( xy0 );

   return doWipe( xy1, xy2, ( xy0.x + xy0.y ) / maxVal );
}

//--------------------------------------------------------------//
float4 Circle_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float scaleFactor = length( float2( 0.5, _Centre.y / _OutputAspectRatio ) );

   xy0 = float2(XPos, YPos) - xy0;
   xy0.y /=_OutputAspectRatio; // Has the effect of making the ellipse circular
   xy0 = frac( float2(XPos, YPos) - ( xy0 * float2( XRepeat, YRepeat ) ) );

   xy0 = pow( xy0 - float2(XPos, YPos), 2 );

   return doWipe( xy1, xy2, sqrt( xy0.x + xy0.y ) / scaleFactor );
}

//--------------------------------------------------------------//
float4 Ellipse_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float halfWidth  = 0.5 * _EllipseSquish;
   float halfHeight = _Centre.y / _OutputAspectRatio;

   xy0 = float2(XPos, YPos) - xy0;
   xy0.y /=_OutputAspectRatio; // Has the effect of making the ellipse circular
   xy0.x *= _EllipseSquish;    // Makes the circle elliptical again, but by a fixed amount which is aspect-ratio independent
   xy0 = frac( float2(XPos, YPos) - ( xy0 * float2( XRepeat, YRepeat ) ) );

   xy0 = pow( xy0 - float2(XPos, YPos), 2 );

   return doWipe( xy1, xy2, sqrt( xy0.x + xy0.y ) / length( float2( halfWidth, halfHeight ) ) );
}
//--------------------------------------------------------------//
float4 Ellipse_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float halfWidth  = 0.5;
   float halfHeight = _Centre.y / _OutputAspectRatio * _EllipseSquish;

   xy0 = float2(XPos, YPos) - xy0;
   xy0.y /=_OutputAspectRatio; // Has the effect of making the ellipse circular
   xy0.y *= _EllipseSquish;    // Makes the circle elliptical again, but by a fixed amount which is aspect-ratio independent
   xy0 = frac( float2(XPos, YPos) - ( xy0 * float2( XRepeat, YRepeat ) ) );

   xy0 = pow( xy0 - float2(XPos, YPos), 2 );

   return doWipe( xy1, xy2, sqrt( xy0.x + xy0.y ) / length( float2( halfWidth, halfHeight ) ) );
}

const float _Tan45 = 1; //tan( 2 * 3.14159 / 8 );
//--------------------------------------------------------------//
float4 Hexagon_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float scaleFactor = length( float2( 0.5, _Centre.y / _OutputAspectRatio ) );

   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 -= float2(XPos, YPos);
   xy0.y /= _OutputAspectRatio; // Effectively makes it square pixels
   xy0 = abs( xy0 );

   float luma;

   if ( ( xy0.x - ( xy0.y / _Tan60 ) ) > 0 )
   {
      luma = ( ( ( _Tan30 * xy0.y ) + xy0.x ) * _Cos30 );
   }
   else
      luma = xy0.y;

   return doWipe( xy1, xy2, luma / scaleFactor );
}

//--------------------------------------------------------------//
/* 4 pointed star

   if ( ( xy0.x - ( xy0.y / _Tan45 ) ) > 0 )
   {
      luma = ( xy0.x / 3 ) + xy0.y;
   }
   else
      luma = ( xy0.y / 3 ) + xy0.x;
*/

//--------------------------------------------------------------//
const float _Tan72  =  3.0777; // tan( 2 * 3.14159 / 5 );
const float _Tan144 = -0.7265; // tan( 2 * 3.14159 / 2.5 );
const float _Tan36  =  0.7265; // tan( 2 * 3.14159 / 10 );
const float _Cos36  =  0.8090; // cos( 2 * 3.14159 / 10 );
const float _Cos72  =  0.3090; // cos( 2 * 3.14159 / 5 );

float4 Hexagon_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float scaleFactor = length( float2( _OutputAspectRatio, 1.0 ) ); // Seems to work, not sure why..
//   float scaleFactor = length( float2( 0.5, _Centre.y / _OutputAspectRatio ) );
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 -= float2(XPos, YPos);
   xy0.y /= _OutputAspectRatio; // Effectively makes it square pixels
   xy0 = abs( xy0 );

   float luma;

   if ( ( xy0.y - ( xy0.x / _Tan60 ) ) > 0 )
   {
      luma = ( ( ( _Tan30 * xy0.x ) + xy0.y ) * _Cos30 );
   }
   else
      luma = xy0.x;

   return doWipe( xy1, xy2, luma * scaleFactor );
}

//-----------------------------------------------------------------------------//
float4 Pentagon_1_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float scaleFactor = length( float2( 0.5, _Centre.y / _OutputAspectRatio ) );
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0 -= float2(XPos, YPos);
   xy0.y /= _OutputAspectRatio; // Effectively makes it square pixels
   xy0.x = abs( xy0.x );

   float luma;

   if ( ( xy0.y - ( xy0.x / _Tan72 ) ) > 0 )
   {
      luma = ( ( _Tan36 * xy0.x ) + xy0.y ) * _Cos36;
   }
   else if ( ( xy0.y - ( xy0.x / _Tan144 ) ) > 0 )
   {
      luma = ( ( _Tan72 * xy0.x ) - xy0.y ) * _Cos72;
   }
   else
   {
      luma = -xy0.y;
   }

   return doWipe( xy1, xy2, luma / scaleFactor );
}

//-----------------------------------------------------------------------------//
float4 Pentagon_2_main( float2 xy0 : TEXCOORD0, float2 xy1 : TEXCOORD1, float2 xy2 : TEXCOORD2 ) : COLOR
{
   float scaleFactor = length( float2( 0.5, _Centre.y / _OutputAspectRatio ) );
   xy0 = frac( float2(XPos, YPos) - ( ( float2(XPos, YPos) - xy0 ) * float2( XRepeat, YRepeat ) ) );

   xy0.y = 1.0 - xy0.y;
   xy0 -= float2(XPos, YPos);
   xy0.y /= _OutputAspectRatio; // Effectively makes it square pixels
   xy0.x = abs( xy0.x );

   float luma;

   if ( ( xy0.y - ( xy0.x / _Tan72 ) ) > 0 )
   {
      luma = ( ( _Tan36 * xy0.x ) + xy0.y ) * _Cos36;
   }
   else if ( ( xy0.y - ( xy0.x / _Tan144 ) ) > 0 )
   {
      luma = ( ( _Tan72 * xy0.x ) - xy0.y ) * _Cos72;
   }
   else
   {
      luma = -xy0.y;
   }

   return doWipe( xy1, xy2, luma / scaleFactor );
}

//-----------------------------------------------------------------------------//
// Techniques
//-----------------------------------------------------------------------------//
technique LinearSampledWipe
{
   pass Pass1 { PixelShader = compile PROFILE linearSampledWipe_main(); }
}

//-----------------------------------------------------------------------------//
technique PointSampledWipe
{
   pass Pass1 { PixelShader = compile PROFILE pointSampledWipe_main(); }
}

technique Bar_1          { pass One { PixelShader = compile PROFILE Bar_1_main(); } }
technique Bar_2          { pass One { PixelShader = compile PROFILE Bar_2_main(); } }
technique Box_1          { pass One { PixelShader = compile PROFILE Box_1_main(); } }
technique Box_2          { pass One { PixelShader = compile PROFILE Box_2_main(); } }
technique Box_3          { pass One { PixelShader = compile PROFILE Box_3_main(); } }
technique Box_4          { pass One { PixelShader = compile PROFILE Box_4_main(); } }
technique Box_5          { pass One { PixelShader = compile PROFILE Box_5_main(); } }
technique Box_6          { pass One { PixelShader = compile PROFILE Box_6_main(); } }
technique Box_7          { pass One { PixelShader = compile PROFILE Box_7_main(); } }
technique Box_8          { pass One { PixelShader = compile PROFILE Box_8_main(); } }
technique Circle         { pass One { PixelShader = compile PROFILE Circle_main(); } }
technique Ellipse_1      { pass One { PixelShader = compile PROFILE Ellipse_1_main(); } }
technique Ellipse_2      { pass One { PixelShader = compile PROFILE Ellipse_2_main(); } }
technique Four_Box_1     { pass One { PixelShader = compile PROFILE Four_Box_1_main(); } }
technique Four_Box_2     { pass One { PixelShader = compile PROFILE Four_Box_2_main(); } }
technique Barn_Door_1    { pass One { PixelShader = compile PROFILE Barn_Door_1_main(); } }
technique Barn_Door_2    { pass One { PixelShader = compile PROFILE Barn_Door_2_main(); } }
technique Barn_Door_3    { pass One { PixelShader = compile PROFILE Barn_Door_3_main(); } }
technique Barn_Door_4    { pass One { PixelShader = compile PROFILE Barn_Door_4_main(); } }
technique Diagonal_1     { pass One { PixelShader = compile PROFILE Diagonal_1_main(); } }
technique Diagonal_2     { pass One { PixelShader = compile PROFILE Diagonal_2_main(); } }
technique Bow_Tie        { pass One { PixelShader = compile PROFILE Bow_Tie_main(); } }
technique Vee_1          { pass One { PixelShader = compile PROFILE Vee_1_main(); } }
technique Vee_2          { pass One { PixelShader = compile PROFILE Vee_2_main(); } }
technique Vee_3          { pass One { PixelShader = compile PROFILE Vee_3_main(); } }
technique Vee_4          { pass One { PixelShader = compile PROFILE Vee_4_main(); } }
technique Barn_Vee_1     { pass One { PixelShader = compile PROFILE Barn_Vee_1_main(); } }
technique Barn_Vee_2     { pass One { PixelShader = compile PROFILE Barn_Vee_2_main(); } }
technique Barn_Vee_3     { pass One { PixelShader = compile PROFILE Barn_Vee_3_main(); } }
technique Barn_Vee_4     { pass One { PixelShader = compile PROFILE Barn_Vee_4_main(); } }
technique Clock_6        { pass One { PixelShader = compile PROFILE Clock_6_main(); } }
technique Clock_12       { pass One { PixelShader = compile PROFILE Clock_12_main(); } }
technique Clock_3        { pass One { PixelShader = compile PROFILE Clock_3_main(); } }
technique Clock_9        { pass One { PixelShader = compile PROFILE Clock_9_main(); } }
technique PinWheel_1     { pass One { PixelShader = compile PROFILE PinWheel_1_main(); } }
technique PinWheel_2     { pass One { PixelShader = compile PROFILE PinWheel_2_main(); } }
technique PinWheel_3     { pass One { PixelShader = compile PROFILE PinWheel_3_main(); } }
technique Single_Sweep_1 { pass One { PixelShader = compile PROFILE Single_Sweep_1_main(); } }
technique Single_Sweep_2 { pass One { PixelShader = compile PROFILE Single_Sweep_2_main(); } }
technique Single_Sweep_3 { pass One { PixelShader = compile PROFILE Single_Sweep_3_main(); } }
technique Single_Sweep_4 { pass One { PixelShader = compile PROFILE Single_Sweep_4_main(); } }
technique Single_Sweep_5 { pass One { PixelShader = compile PROFILE Single_Sweep_5_main(); } }
technique Single_Sweep_6 { pass One { PixelShader = compile PROFILE Single_Sweep_6_main(); } }
technique Single_Sweep_7 { pass One { PixelShader = compile PROFILE Single_Sweep_7_main(); } }
technique Single_Sweep_8 { pass One { PixelShader = compile PROFILE Single_Sweep_8_main(); } }
technique Double_Sweep_1 { pass One { PixelShader = compile PROFILE Double_Sweep_1_main(); } }
technique Double_Sweep_2 { pass One { PixelShader = compile PROFILE Double_Sweep_2_main(); } }
technique Double_Sweep_3 { pass One { PixelShader = compile PROFILE Double_Sweep_3_main(); } }
technique Double_Sweep_4 { pass One { PixelShader = compile PROFILE Double_Sweep_4_main(); } }
technique Double_Sweep_5 { pass One { PixelShader = compile PROFILE Double_Sweep_5_main(); } }
technique Double_Sweep_6 { pass One { PixelShader = compile PROFILE Double_Sweep_6_main(); } }
technique Saloon_Door_1  { pass One { PixelShader = compile PROFILE Saloon_Door_1_main(); } }
technique Saloon_Door_2  { pass One { PixelShader = compile PROFILE Saloon_Door_2_main(); } }
technique Saloon_Door_3  { pass One { PixelShader = compile PROFILE Saloon_Door_3_main(); } }
technique Saloon_Door_4  { pass One { PixelShader = compile PROFILE Saloon_Door_4_main(); } }
technique Windshield_1   { pass One { PixelShader = compile PROFILE Windshield_1_main(); } }
technique Windshield_2   { pass One { PixelShader = compile PROFILE Windshield_2_main(); } }
technique Windshield_3   { pass One { PixelShader = compile PROFILE Windshield_3_main(); } }
technique Windshield_4   { pass One { PixelShader = compile PROFILE Windshield_4_main(); } }
technique Double_Fan_1   { pass One { PixelShader = compile PROFILE Double_Fan_1_main(); } }
technique Fan_1          { pass One { PixelShader = compile PROFILE Fan_1_main(); } }
technique Fan_2          { pass One { PixelShader = compile PROFILE Fan_2_main(); } }
technique Fan_3          { pass One { PixelShader = compile PROFILE Fan_3_main(); } }
technique Fan_4          { pass One { PixelShader = compile PROFILE Fan_4_main(); } }
technique Fan_5          { pass One { PixelShader = compile PROFILE Fan_5_main(); } }
technique Fan_6          { pass One { PixelShader = compile PROFILE Fan_6_main(); } }
technique Square         { pass One { PixelShader = compile PROFILE Square_main(); } }
technique Diamond        { pass One { PixelShader = compile PROFILE Diamond_main(); } }
technique Hexagon_1      { pass One { PixelShader = compile PROFILE Hexagon_1_main(); } }
technique Hexagon_2      { pass One { PixelShader = compile PROFILE Hexagon_2_main(); } }
technique Pentagon_1     { pass One { PixelShader = compile PROFILE Pentagon_1_main(); } }
technique Pentagon_2     { pass One { PixelShader = compile PROFILE Pentagon_2_main(); } }
