//--------------------------------------------------------------//
// BoxBlur
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Two Tone";
   string Category    = "Colour Effects";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
float Level
<
   string Description = "Threshold";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.5;

float4 DarkColour
<
   string Description = "Dark Colour";
> = { 0.0, 0.0, 0.0, 1.0 };

float4 LightColour
<
   string Description = "Light Colour";
> = { 1.0, 1.0, 1.0, 1.0 };

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture Input;
sampler InputSampler = sampler_state
{
   Texture = <Input>;
};

texture ThresholdTexture : RenderColorTarget < float2 ViewportRatio={1.0,1.0}; >;
sampler ThresholdSampler = sampler_state
{
   Texture = <ThresholdTexture>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;
};

texture Blur1 : RenderColorTarget < float2 ViewportRatio={1.0,1.0}; >;
sampler BlurSampler = sampler_state
{
   Texture = <Blur1>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;
};

float _OutputWidth  = 1.0;
float _OutputHeight = 1.0;

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 threshold_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src1 = tex2D( InputSampler, xy1 );
   float srcLum = ( ( src1.r * 0.3 ) + ( src1.g * 0.59 ) + ( src1.b * 0.11 ) );

   if ( srcLum > Level )
      src1.rgb = LightColour.rgb;
   else
      src1.rgb = DarkColour.rgb;

   return src1;
}
//--------------------------------------------------------------//
float4 blurX_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float one   = 1.0 / _OutputWidth;
   float tap1  = xy1.x + one;
   float ntap1 = xy1.x - one;

   float4 ret =  tex2D( ThresholdSampler, xy1 );
   float4 blurred = ret;

   blurred += tex2D( ThresholdSampler, float2( tap1,  xy1.y ) );
   blurred += tex2D( ThresholdSampler, float2( ntap1, xy1.y ) );
   blurred /= 3;

   return blurred;
}

//--------------------------------------------------------------//
float4 blurY_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float one  = 1.0 / _OutputHeight;
   float tap1 = xy1.y + one;
   float ntap1 = xy1.y - one;

   float4 ret    = tex2D( BlurSampler, xy1 );
   float4 source = tex2D( InputSampler, xy1 );
   ret += tex2D( BlurSampler, float2( xy1.x, tap1 ) );
   ret += tex2D( BlurSampler, float2( xy1.x, ntap1 ) );
   ret /= 3;

   ret = lerp( source, ret, source.a );
   ret.a = source.a;

   return ret;
}

//--------------------------------------------------------------//
technique Threshold
{
   pass ThresholdPass
   <
      string Script = "RenderColorTarget0 = ThresholdTexture;";
   >
   {
      PixelShader = compile PROFILE threshold_main();
   }

   pass BlurX
   <
      string Script = "RenderColorTarget0 = Blur1;";
   >
   {
      PixelShader = compile PROFILE blurX_main();
   }

   pass BlurY
   {
      PixelShader = compile PROFILE blurY_main();
   }
}
