//--------------------------------------------------------------//
// BoxBlur
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Tri Tone";
   string Category    = "Colour Effects";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//
float Level1
<
   string Description = "Threshold One";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.33;

float Level2
<
   string Description = "Threshold Two";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.66;

float4 DarkColour
<
   string Description = "Dark Colour";
> = { 0.0, 0.0, 0.0, 1.0 };

float4 MidColour
<
   string Description = "Mid Colour";
> = { 0.5, 0.5, 0.5, 1.0 };

float4 LightColour
<
   string Description = "Light Colour";
> = { 1.0, 1.0, 1.0, 1.0 };

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture Input;
sampler InputSampler = sampler_state
{
   Texture = <Input>;
};

texture ThresholdTexture : RenderColorTarget;
sampler ThresholdSampler = sampler_state
{
   Texture = <ThresholdTexture>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;
};

texture Blur1 : RenderColorTarget;
sampler BlurSampler = sampler_state
{
   Texture = <Blur1>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;
};

float _OutputWidth  = 1.0;
float _OutputHeight = 1.0;

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//
float4 threshold_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float4 src1 = tex2D( InputSampler, xy1 );
   float srcLum = ( ( src1.r * 0.3 ) + ( src1.g * 0.59 ) + ( src1.b * 0.11 ) );

   if ( srcLum < Level1 )
      src1.rgb = DarkColour.rgb;
   else if ( srcLum < Level2 )
      src1.rgb = MidColour.rgb;
   else
      src1.rgb = LightColour.rgb;

   return src1;
}

const float blur[] = { 20.0 / 64.0, 15.0 / 64.0, 6.0  / 64.0, 1.0  / 64.0 };  // See Pascals Triangle
//const float blur[] = { 4.0 / 12.0,  3.0 / 12.0,  1.0 / 12.0 };  // See Pascals Triangle
//const float blur[] = { 2.0 / 4.0,  1.0 / 4.0 };  // See Pascals Triangle

//--------------------------------------------------------------//
// Blur1
//--------------------------------------------------------------//
float4 blur1_ps_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   // Explicitly query BETWEEN pixels to get extra averaging
   float2 onePixAcross   = float2( 0.5 / _OutputWidth, 0.0 );
   float2 twoPixAcross   = float2( 1.5 / _OutputWidth, 0.0 );
   float2 threePixAcross = float2( 2.5 / _OutputWidth, 0.0 );

   float4 keyPix = tex2D( ThresholdSampler, xy1 );

   float4 result = keyPix * blur[ 0 ];
   result += tex2D( ThresholdSampler, xy1 + onePixAcross )   * blur[ 1 ];
   result += tex2D( ThresholdSampler, xy1 - onePixAcross )   * blur[ 1 ];
   result += tex2D( ThresholdSampler, xy1 + twoPixAcross )   * blur[ 2 ];
   result += tex2D( ThresholdSampler, xy1 - twoPixAcross )   * blur[ 2 ];
   result += tex2D( ThresholdSampler, xy1 + threePixAcross ) * blur[ 3 ];
   result += tex2D( ThresholdSampler, xy1 - threePixAcross ) * blur[ 3 ];
   result.a = keyPix.a;

   return result;
}


//--------------------------------------------------------------//
// Blur2
//--------------------------------------------------------------//
float4 blur2_ps_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   // Explicitly query BETWEEN pixels to get extra averaging
   float2 onePixDown   = float2( 0.0, 0.5 / _OutputHeight );
   float2 twoPixDown   = float2( 0.0, 1.5 / _OutputHeight );
   float2 threePixDown = float2( 0.0, 2.5 / _OutputHeight );

   float4 keyPix = tex2D( BlurSampler, xy1 );
   float4 source = tex2D( InputSampler, xy1 );

   float4 result = keyPix * blur[ 0 ];
   result += tex2D( BlurSampler, xy1 + onePixDown )   * blur[ 1 ];
   result += tex2D( BlurSampler, xy1 - onePixDown )   * blur[ 1 ];
   result += tex2D( BlurSampler, xy1 + twoPixDown )   * blur[ 2 ];
   result += tex2D( BlurSampler, xy1 - twoPixDown )   * blur[ 2 ];
   result += tex2D( BlurSampler, xy1 + threePixDown ) * blur[ 3 ];
   result += tex2D( BlurSampler, xy1 - threePixDown ) * blur[ 3 ];
   result.a = keyPix.a;
/*
   float resultLum = ( ( result.r * 0.3 ) + ( result.g * 0.59 ) + ( result.b * 0.11 ) );

   if ( resultLum < Level1 )
      result.rgb = DarkColour.rgb;
   else if ( resultLum < Level2 )
      result.rgb = MidColour.rgb;
   else
      result.rgb = LightColour.rgb;
*/

   result = lerp( source, result, source.a );
   result.a = source.a;

   return result;
}

//--------------------------------------------------------------//
technique BoxBlur
{
   pass ThresholdPass
   <
      string Script = "RenderColorTarget0 = ThresholdTexture;";
   >
   {
      PixelShader = compile PROFILE threshold_main();
   }

   pass BlurX
   <
      string Script = "RenderColorTarget0 = Blur1;";
   >
   {
      PixelShader = compile PROFILE blur1_ps_main();
   }

   pass BlurY
   {
      PixelShader = compile PROFILE blur2_ps_main();
   }
}
