//--------------------------------------------------------------//
// Unsharp Mask
//
// Copyright (c) EditShare EMEA.  All Rights Reserved
//--------------------------------------------------------------//

int _LwksEffectInfo
<
   string EffectGroup = "GenericPixelShader";
   string Description = "Sharpen Edges";
   string Category    = "Stylize";
> = 0;

//--------------------------------------------------------------//
// Params
//--------------------------------------------------------------//

float Radius
<
   string Description = "Radius";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 1.0;

float Strength
<
   string Description = "Strength";
   float MinVal = 0.00;
   float MaxVal = 5.00;
> = 1.0;

float Tolerance
<
   string Description = "Tolerance";
   float MinVal = 0.00;
   float MaxVal = 1.00;
> = 0.8;

float _OutputWidth  = 1.0;
float _OutputHeight = 1.0;

//--------------------------------------------------------------//
// Inputs
//--------------------------------------------------------------//
texture Input;
sampler InputSampler = sampler_state
{
   Texture = <Input>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;
   AddressU = ClampToEdge;
   AddressV = ClampToEdge;
};

texture Blur1 : RenderColorTarget;

sampler BlurSampler = sampler_state
{
   Texture = <Blur1>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;
};

texture Blur2 : RenderColorTarget;

sampler Blur2Sampler = sampler_state
{
   Texture = <Blur2>;
   MINFILTER = LINEAR;
   MIPFILTER = LINEAR;
   MAGFILTER = LINEAR;
};

//--------------------------------------------------------------//
// Code
//--------------------------------------------------------------//

const float gaussian[] = { 1.0 / 4096.0, 12.0 / 4096.0, 66.0 / 4096.0, 220.0 / 4096.0, 495.0 / 4096.0, 792.0 / 4096.0, 924.0 / 4096.0 };

float4 blurX_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float one       = Radius / _OutputWidth;
   float tap_2_3   = xy1.x + one;
   float tap_4_5   = tap_2_3 + one;
   float tap_6_7   = tap_4_5 + one;;
   float tap_8_9   = tap_6_7 + one;
   float tap_10_11  = tap_8_9 + one;
   float tap_12_13 = tap_10_11 + one;
   float ntap_2_3   = xy1.x - one;
   float ntap_4_5   = ntap_2_3 - one;
   float ntap_6_7   = ntap_4_5 - one;;
   float ntap_8_9   = ntap_6_7 - one;
   float ntap_10_11 = ntap_8_9 - one;
   float ntap_12_13 = ntap_10_11 - one;

   float4 ret =  tex2D( InputSampler, xy1 ) * gaussian[ 6 ];
   float4 blurred = ret;

   blurred += tex2D( InputSampler, float2( tap_2_3,    xy1.y ) ) * gaussian[ 5 ];
   blurred += tex2D( InputSampler, float2( tap_4_5,    xy1.y ) ) * gaussian[ 4 ];
   blurred += tex2D( InputSampler, float2( tap_6_7,    xy1.y ) ) * gaussian[ 3 ];
   blurred += tex2D( InputSampler, float2( tap_8_9,    xy1.y ) ) * gaussian[ 2 ];
   blurred += tex2D( InputSampler, float2( tap_10_11,  xy1.y ) ) * gaussian[ 1 ];
   blurred += tex2D( InputSampler, float2( tap_12_13,  xy1.y ) ) * gaussian[ 0 ];

   blurred += tex2D( InputSampler, float2( ntap_2_3,   xy1.y ) ) * gaussian[ 5 ];
   blurred += tex2D( InputSampler, float2( ntap_4_5,   xy1.y ) ) * gaussian[ 4 ];
   blurred += tex2D( InputSampler, float2( ntap_6_7,   xy1.y ) ) * gaussian[ 3 ];
   blurred += tex2D( InputSampler, float2( ntap_8_9,   xy1.y ) ) * gaussian[ 2 ];
   blurred += tex2D( InputSampler, float2( ntap_10_11, xy1.y ) ) * gaussian[ 1 ];
   blurred += tex2D( InputSampler, float2( ntap_12_13, xy1.y ) ) * gaussian[ 0 ];

   return blurred;
}

float4 blurY_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float one       = Radius / _OutputHeight;
   float tap_2_3   = xy1.y + one;
   float tap_4_5   = tap_2_3 + one;
   float tap_6_7   = tap_4_5 + one;;
   float tap_8_9   = tap_6_7 + one;
   float tap_10_11  = tap_8_9 + one;
   float tap_12_13 = tap_10_11 + one;

   float ntap_2_3   = xy1.y - one;
   float ntap_4_5   = ntap_2_3 - one;
   float ntap_6_7   = ntap_4_5 - one;;
   float ntap_8_9   = ntap_6_7 - one;
   float ntap_10_11 = ntap_8_9 - one;
   float ntap_12_13 = ntap_10_11 - one;

   float4 ret =  tex2D( BlurSampler, xy1 ) * gaussian[ 6 ];
   float4 blurred = ret;
   blurred += tex2D( BlurSampler, float2( xy1.x, tap_2_3    ) ) * gaussian[ 5 ];
   blurred += tex2D( BlurSampler, float2( xy1.x, tap_4_5    ) ) * gaussian[ 4 ];
   blurred += tex2D( BlurSampler, float2( xy1.x, tap_6_7    ) ) * gaussian[ 3 ];
   blurred += tex2D( BlurSampler, float2( xy1.x, tap_8_9    ) ) * gaussian[ 2 ];
   blurred += tex2D( BlurSampler, float2( xy1.x, tap_10_11  ) ) * gaussian[ 1 ];
   blurred += tex2D( BlurSampler, float2( xy1.x, tap_12_13  ) ) * gaussian[ 0 ];

   blurred += tex2D( BlurSampler, float2( xy1.x, ntap_2_3   ) ) * gaussian[ 5 ];
   blurred += tex2D( BlurSampler, float2( xy1.x, ntap_4_5   ) ) * gaussian[ 4 ];
   blurred += tex2D( BlurSampler, float2( xy1.x, ntap_6_7   ) ) * gaussian[ 3 ];
   blurred += tex2D( BlurSampler, float2( xy1.x, ntap_8_9   ) ) * gaussian[ 2 ];
   blurred += tex2D( BlurSampler, float2( xy1.x, ntap_10_11 ) ) * gaussian[ 1 ];
   blurred += tex2D( BlurSampler, float2( xy1.x, ntap_12_13 ) ) * gaussian[ 0 ];

   return blurred;
}

float4 maths_main( float2 xy1 : TEXCOORD1 ) : COLOR
{
   float threshold  = 1.0 - Tolerance;
   float4 orig      = tex2D( InputSampler, xy1 );
   float4 blurred   = tex2D( Blur2Sampler, xy1 );
   float4 sharpened = orig + ( Strength * ( orig - blurred ) );

   float dist = distance( sharpened, orig );

   if ( dist <= threshold )
   {
      float softThresh = threshold - 0.1;

      if ( dist > softThresh )
      {
         sharpened = lerp( orig, sharpened, ( dist - softThresh ) / 0.1 );
      }
      else
         sharpened = orig;
   }

   sharpened = lerp( orig, sharpened, orig.a );
   sharpened.a = orig.a;

   return sharpened;
}

technique GaussianBlur
{
   pass BlurX
   <
      string Script = "RenderColorTarget0 = Blur1;";
   >
   {
      PixelShader = compile PROFILE blurX_main();
   }

   pass BlurY
   <
      string Script = "RenderColorTarget0 = Blur2;";
   >
   {
      PixelShader = compile PROFILE blurY_main();
   }

   pass Maths
   {
      PixelShader = compile PROFILE maths_main();
   }
}

